from panther_core import PantherEvent

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class OrcaPassthrough(Rule):
    id = "Orca.Passthrough-prototype"
    display_name = "Orca Passthrough"
    log_types = [LogType.ORCA_ALERT]
    default_severity = Severity.MEDIUM
    default_description = "Re-raise Orca alerts in Panther\n"
    dedup_period_minutes = 1440
    default_reference = "https://docs.orcasecurity.io/docs/alerts\n"
    summary_attributes = ["alert_labels", "asset_name", "state:alert_id"]
    tags = ["Orca", "Passthrough"]

    def rule(self, event) -> bool:
        return event.deep_get("state", "status") == "open"

    def title(self, event: PantherEvent) -> str:
        alert_type = event.get("asset_type_string")
        # Use the first non-null field for the title of the alert
        alert_desc = event.deep_get("data", "title") or event.get("type_string") or "<UNKNOWN ALERT TITLE>"
        return f"{alert_type}: {alert_desc}" if alert_type else alert_desc

    def description(self, event):
        return event.get("description") or "DEFAULT"

    def dedup(self, event: PantherEvent) -> str:
        # Explicitly dedup on severity
        return f"({self.severity(event)}) {self.title(event)}"

    def severity(self, event: PantherEvent) -> str:
        match event.deep_get("state", "risk_level", default="medium"):
            case "informational":
                return "INFO"
            case "low":
                return "LOW"
            case "medium":
                return "MEDIUM"
            case "high":
                return "HIGH"
            case "critical":
                return "CRITICAL"
            case _:
                return "DEFAULT"

    def alert_context(self, event: PantherEvent) -> dict:
        return {
            "asset": {
                "name": event.get("asset_name", "<UNKNOWN ASSET NAME>"),
                "type": event.get("asset_type", "<UNKNOWN TYPE>"),
                "category": event.get("asset_category", "<UNKNOWN CATEGORY>"),
            },
            "category": event.get("category", "<UNKNOWN CATEGORY>"),
            "cloud_provider": {
                "id": event.get("cloud_provider_id", "<UNKNOWN_CLOUD_PROVIDER_ID>"),
                "type": event.get("cloud_provider", "<UNKNOWN_CLOUD_PROVIDER>"),
            },
            "details": event.deep_get("data", "details", default=""),
            "orca_alert_id": event.deep_get("state", "alert_id", default="<UNKNOWN ALERT ID>"),
            "org_id": event.get("organization_id", "<UNKNOWN ORG ID>"),
        }

    def runbook(self, event: PantherEvent) -> dict:
        return event.get("recommendation") or event.deep_get("data", "recommendation", default="")

    tests = [
        RuleTest(
            name="Repo Alert - Use of Weak Hash Algorithm",
            expected_result=True,
            log={
                "p_event_time": "2025-03-13 07:37:05.000000000",
                "p_log_type": "Orca.Alert",
                "p_parse_time": "2025-03-13 07:45:02.263118919",
                "p_row_id": "2adeaf6dac909095c0feb0a525b6d902",
                "p_schema_version": 0,
                "p_source_id": "d0904c6a-e437-4082-8807-6bdccc0fc16f",
                "p_source_label": "Orca Prod",
                "account_name": "org-name/repo-name (Project: org-name/repo-name)",
                "alert_labels": ["CWE-327", "shiftleft:sast", "shiftleft:sast:lang:java", "source:shiftleft"],
                "alert_source": "Orca Scan",
                "asset_category": "CI Source",
                "asset_name": "org-name/repo-name",
                "asset_state": "enabled",
                "asset_type": "cisource",
                "asset_type_string": "CodeRepository",
                "asset_unique_id": "CodeRepository_32fe97a4-ec58-4823-9521-b226fd4f51a7_e739eb76-0ae6-624a-6479-d00bc1fe37c1",
                "category": "Best practices",
                "cloud_account_id": "fd01de1b-e355-4d9c-8a0f-142dee95ae6f",
                "cloud_account_type": "Regular",
                "cloud_provider": "shiftleft",
                "cloud_provider_id": "32fe97a4-ec58-4823-9521-b226fd4f51a7",
                "cloud_vendor_id": "32fe97a4-ec58-4823-9521-b226fd4f51a7",
                "cluster_name": "org-name/repo-name",
                "cluster_type": "cisource",
                "cluster_unique_id": "CodeRepository_32fe97a4-ec58-4823-9521-b226fd4f51a7_e739eb76-0ae6-624a-6479-d00bc1fe37c1",
                "configuration": {},
                "container_id": "coderepository-a026f1a3-7ad4-492f-a9ef-b4213f05eba2-fcd577d6-99d4-4489-9f24-03b45bb6a43c",
                "context": "data",
                "description": "The rule detects the use of weak hash algorithms that are vulnerable to hash collisions. Using these algorithms can lead to security risks, including data integrity issues. It is crucial to adopt stronger hash functions to maintain secure cryptographic practices.",
                "details": "The rule detects the use of weak hash algorithms that are vulnerable to hash collisions. Using these algorithms can lead to security risks, including data integrity issues. It is crucial to adopt stronger hash functions to maintain secure cryptographic practices.",
                "group_name": "org-name/repo-name",
                "group_type": "cisource",
                "group_type_string": "NonGroup",
                "group_unique_id": "CodeRepository_32fe97a4-ec58-4823-9521-b226fd4f51a7_e739eb76-0ae6-624a-6479-d00bc1fe37c1",
                "group_val": "nongroup",
                "is_compliance": False,
                "level": 0,
                "organization_id": "5f1c59f5-7c6c-4a56-a6ff-94998abdf9c3",
                "organization_name": "Partners",
                "recommendation": "Replace weak hash algorithms with stronger alternatives such as SHA-256 or SHA-3 to prevent vulnerabilities related to hash collisions and enhance security.",
                "rule_id": "rc72e2d8cf8",
                "source": "src/main/java/org/owasp/webgoat/lessons/cryptography/HashingAssignment.java:55",
                "state": {
                    "alert_id": "orca-123456",
                    "created_at": "2025-03-13 07:37:05.000000000",
                    "high_since": "2025-03-13 07:38:34.000000000",
                    "in_verification": False,
                    "last_seen": "2025-03-13 07:37:05.000000000",
                    "last_updated": "2025-03-13 07:37:05.000000000",
                    "orca_score": 5.8,
                    "risk_level": "medium",
                    "rule_source": "Orca",
                    "score": 3,
                    "severity": "hazardous",
                    "status": "open",
                    "status_time": "2025-03-13 07:37:05.000000000",
                },
                "subject_type": "CodeRepository_32fe97a4-ec58-4823-9521-b226fd4f51a7_e739eb76-0ae6-624a-6479-d00bc1fe37c1",
                "type": "shiftleft_96f8e29d-f941-5559-96e8-0ada0431b3a1",
                "type_key": "6d9583cc014beabccfe02c0debb64a28b9d73cc4ef87079fdfb2e23b2d9d8c8d6843e21a79f0059d62eadcef5fb5af0da3ad72c7406d688c544d51af6e62396d_0",
                "type_string": "Use of Weak Hash Algorithms in Cryptography",
            },
        ),
        RuleTest(
            name="AWS Network Alert - All Outbound Traffic Allowed",
            expected_result=True,
            log={
                "p_any_aws_account_ids": ["111111111111"],
                "p_event_time": "2025-03-13 04:05:33.000000000",
                "p_log_type": "Orca.Alert",
                "p_parse_time": "2025-03-13 05:15:02.300809961",
                "p_row_id": "7a260fc49c66a2cf988699a5259d09",
                "p_schema_version": 0,
                "p_source_id": "d0904c6a-e437-4082-8807-6bdccc0fc16f",
                "p_source_label": "Orca Prod",
                "p_udm": {},
                "account_name": "111111111111",
                "alert_labels": ["mitre: exfiltration", "CSPM"],
                "alert_source": "Orca Scan",
                "asset_category": "Network",
                "asset_name": "acl-0123456789abcdef0",
                "asset_regions": ["ap-southeast-2"],
                "asset_state": "enabled",
                "asset_type": "AwsEc2NetworkAcl",
                "asset_type_string": "AwsEc2NetworkAcl",
                "asset_unique_id": "AwsEc2NetworkAcl_111111111111_e739eb76-8bac-06e9-b9b3-07ef41f75278",
                "asset_vendor_id": "acl-0123456789abcdef0",
                "category": "Network misconfigurations",
                "cloud_account_id": "5f1c59f5-7c6c-4a56-a6ff-94998abdf9c3",
                "cloud_account_type": "Regular",
                "cloud_provider": "aws",
                "cloud_provider_id": "111111111111",
                "cloud_vendor_id": "111111111111",
                "cluster_name": "acl-0123456789abcdef0",
                "cluster_type": "AwsEc2NetworkAcl",
                "cluster_unique_id": "AwsEc2NetworkAcl_111111111111_e739eb76-8bac-06e9-b9b3-07ef41f75278",
                "configuration": {},
                "context": "control",
                "data": {
                    "details": "Ensure there are no Network ACLs with Outbound rule to allow All ICMP IPv4",
                    "mitre_category": "exfiltration",
                    "mitre_technique": ["Transfer Data to Cloud Account (T1537)"],
                    "mitre_techniques": ["Transfer Data to Cloud Account (T1537)"],
                    "recommendation": "Edit the Network ACLs outbound rules",
                    "remediation_console": [
                        ">1. Sign in to the AWS Management Console and open the **[VPC console](https://console.aws.amazon.com/vpc)**.",
                        ">2. On the navigation pane, under the **SECURITY** section, select **Network ACLs**.",
                        ">3. On the bottom pane ,select the **Outbound rules** and click on **Edit outbound rules**.",
                        ">4. Review the rules which their type is **All ICMP IPV4**.",
                        ">5. Review the value in the **Source** column and make sure that it's not 0.0.0.0/0. Make sure **Allow/Deny** attribute is set to **Allow**.",
                    ],
                    "title": "Network ACL Allow all Outbound ICMP IPv4 Traffic",
                },
                "description": "Network ACL Allow all Outbound ICMP IPv4 Traffic",
                "details": "Ensure there are no Network ACLs with Outbound rule to allow All ICMP IPv4",
                "findings": [
                    {
                        "Asset Risk Score": "1.0",
                        "Association IDs": ["aclassoc-07bf2bba155bf79c3"],
                        "Category": "Network",
                        "Category (New)": "Network",
                        "Exposure": "N/A",
                        "First Seen": "2025-03-13T04:05:45+00:00",
                        "Is Default": "Yes",
                        "Last Seen": "2025-03-13T04:05:45+00:00",
                        "Name": "acl-0123456789abcdef0",
                        "Network ACL ID": "acl-0123456789abcdef0",
                        "Network Acl Rules": ["acl-0123456789abcdef0_0"],
                        "Publicly Allowed Admin Ports": [22, 3389],
                        "Region": "ap-southeast-2",
                        "Sub Category": "Network Security",
                        "SubCategory (New)": "Segregation/Transport",
                        "Subnets": ["my_subnet"],
                        "UI Unique Field": "acl-0123456789abcdef0",
                        "VPC": "my_vpc",
                    },
                ],
                "group_name": "acl-0123456789abcdef0",
                "group_type": "AwsEc2NetworkAcl",
                "group_type_string": "NonGroup",
                "group_unique_id": "AwsEc2NetworkAcl_111111111111_e739eb76-8bac-06e9-b9b3-07ef41f75278",
                "group_val": "nongroup",
                "is_compliance": False,
                "is_rule": True,
                "level": 0,
                "organization_id": "5f1c59f5-7c6c-4a56-a6ff-94998abdf9c3",
                "organization_name": "Partners",
                "recommendation": "Edit the Network ACLs outbound rules",
                "related_compliances": [
                    "AWS Well-Architected",
                    "CSA CCM Lite v4.0.13",
                    "CSA CCM v4.0.5",
                    "FedRAMP",
                    "HITRUST Level 3 v9.6.0",
                    "ISM (Australian Government Information Security Manual) September 2022",
                    "ISO 27001 2022",
                    "ISO 27002 2022",
                    "MITRE ATT&CK v12",
                    "MITRE ATT&CK v13",
                    "NIST 800-171 (Rev 3)",
                    "NIST 800-53 (Rev 5)",
                    "NIST 800-53 (Rev 5.1.1)",
                    "Orca Best Practices",
                    "Orca Best Practices 2.0.0",
                    "PCI DSS 3.2.1",
                    "PCI DSS 4.0.0",
                    "UK Cyber Essentials",
                    "UK NCSC CAF (National Cyber Security Centre Cyber Assessment Framework)",
                    "US CJIS (Criminal Justice Information Services) 6.0.0",
                    "US CMMC (Cybersecurity Maturity Model Certification) Level 2",
                    "US FFIEC (Federal Financial Institutions Examination Council)",
                    "US MARS-E Volume 1 2.2.0",
                ],
                "rule_id": "r3832181aab",
                "rule_query": "AwsEc2NetworkAcl with NetworkAclRules with Egress and (Protocol = '-1' or Protocol = '1' or Protocol = 'icmp') and CidrBlock = '0.0.0.0/0' and RuleAction = 'allow'",
                "source": "acl-0123456789abcdef0",
                "state": {
                    "alert_id": "orca-123456",
                    "created_at": "2025-03-13 04:05:33.000000000",
                    "in_verification": False,
                    "last_seen": "2025-03-13 04:05:33.000000000",
                    "last_updated": "2025-03-13 04:05:33.000000000",
                    "low_since": "2025-03-13 05:06:09.000000000",
                    "orca_score": 3.5,
                    "risk_level": "low",
                    "rule_source": "Orca",
                    "score": 4,
                    "severity": "informational",
                    "status": "open",
                    "status_time": "2025-03-13 04:05:33.000000000",
                },
                "subject_type": "AwsEc2NetworkAcl",
                "type": "nacl_allows_all_outbound_ipv4_icmp",
                "type_key": "68e26fa3629dc0203ceeb0b1d3c5b185",
                "type_string": "Network ACL Allow all Outbound ICMP IPv4 Traffic",
            },
        ),
        RuleTest(
            name="AWS EC2 Alert - SSH Logins",
            expected_result=True,
            log={
                "p_any_ip_addresses": ["1.1.1.1", "2.2.2.2"],
                "p_any_aws_account_ids": ["111111111111"],
                "p_any_domain_names": ["ip-2-2-2-2"],
                "p_event_time": "2025-03-13 04:05:33.000000000",
                "p_log_type": "Orca.Alert",
                "p_parse_time": "2025-03-13 06:05:02.201708487",
                "p_row_id": "7a260fc49c66a2cf988699a525c5e102",
                "p_schema_version": 0,
                "p_source_id": "d0904c6a-e437-4082-8807-6bdccc0fc16f",
                "p_source_label": "Orca Prod",
                "p_udm": {},
                "account_name": "111111111111",
                "alert_labels": ["mitre: credential access", "OWASP: NHI4:2025"],
                "alert_source": "Orca Scan",
                "asset_auto_updates": "on",
                "asset_availability_zones": ["ap-southeast-2b"],
                "asset_category": "VM",
                "asset_distribution_major_version": "20",
                "asset_distribution_name": "Ubuntu",
                "asset_distribution_version": "20.04",
                "asset_first_private_dnss": ["ip-2-2-2-2.ap-southeast-2.compute.internal"],
                "asset_first_private_ips": ["2.2.2.2"],
                "asset_first_public_ips": ["1.1.1.1"],
                "asset_hostname": "ip-2-2-2-2",
                "asset_image_id": "ami-0123456789abcdef",
                "asset_ingress_ports": ["20-22"],
                "asset_labels": ["internet_facing"],
                "asset_name": "my_ec2",
                "asset_num_private_dnss": 1,
                "asset_num_private_ips": 1,
                "asset_num_public_ips": 1,
                "asset_regions": ["ap-southeast-2"],
                "asset_regions_names": ["Sydney"],
                "asset_role_names": ["ssh"],
                "asset_state": "running",
                "asset_tags_info_list": ["Name|my_ec2"],
                "asset_type": "vm",
                "asset_type_string": "VM",
                "asset_unique_id": "vm_111111111111_i-abcdef01234567890",
                "asset_vendor_id": "i-abcdef01234567890",
                "asset_vpcs": ["vpc-abcdef01234567890"],
                "category": "Authentication",
                "cloud_account_id": "5f1c59f5-7c6c-4a56-a6ff-94998abdf9c3",
                "cloud_account_type": "Regular",
                "cloud_provider": "aws",
                "cloud_provider_id": "111111111111",
                "cloud_vendor_id": "111111111111",
                "cluster_name": "my_ec2",
                "cluster_type": "vm",
                "cluster_unique_id": "vm_111111111111_i-abcdef01234567890",
                "configuration": {},
                "container_id": "main",
                "context": "data",
                "data": {
                    "details": "The SSH configuration on the operation system is set to allow username and password authentication. SSH authentication based on username and password is vulnerable to brute force attacks, which could result in asset compromise if successful.",
                    "mitre_category": "credential access",
                    "mitre_technique": ["Brute Force (T1110)"],
                    "mitre_techniques": ["Brute Force (T1110)"],
                    "recommendation": 'It is recommended to disable SSH password authentication by editing the SSH configuration file (/etc/ssh/sshd_config) from this line "PasswordAuthentication yes" to "PasswordAuthentication no" and enable key-pair authentication to secure SSH authentication process',
                    "remediation_cli": [
                        ">- Edit the SSH configurations file **/etc/ssh/sshd_config** and set the below parameter:",
                        ">**`PasswordAuthentication no`**",
                    ],
                    "title": "SSH Authentication with username and password",
                },
                "description": "SSH Authentication with username and password",
                "details": "The SSH configuration on the operation system is set to allow username and password authentication. SSH authentication based on username and password is vulnerable to brute force attacks, which could result in asset compromise if successful.",
                "findings": [
                    {
                        "Applications": ["EC2 Instance Connect", "SQLite", "Rsyslog", "OpenSSH Server"],
                        "Asset Risk Score": 1,
                        "Auto Updates Status": "on",
                        "Availability Zones": ["ap-southeast-2b"],
                        "CPU Count": 1,
                        "CPU Family": "Intel",
                        "CPU Frequency": 2300,
                        "CPU Type": "Intel(R) Xeon(R) CPU E5-2686 v4",
                        "CVE Count": '{"In-2023": 0, "In-2024": 3, "In-2025": 4, "LastYear": 3, "TwoYearsAgo": 0}',
                        "Category": "VM",
                        "Category (New)": "Compute Services",
                        "Console URL Link": "https://ap-southeast-2.console.aws.amazon.com/ec2/home?region=ap-southeast-2#InstanceDetails:instanceId=i-abcdef01234567890",
                        "Creation Time": "2025-03-12T13:55:32+00:00",
                        "Disks": [
                            "{'size_bytes': 8589934592, 'size': '8.0 GB', 'used_bytes': 1916348416, 'used': '1.78 GB', 'volume_id': 'vol-04e00a684c0e419be'}",
                        ],
                        "Distribution Info Site": "https://ubuntu.com/",
                        "Distribution Major Version": "20",
                        "Distribution Name": "Ubuntu",
                        "Distribution Version": "20.04",
                        "Distro Category": "debian",
                        "ESM Status": "disabled",
                        "Exposure": "internet_facing",
                        "First Seen": "2025-03-13T04:05:49+00:00",
                        "Has Malware With High Confidence": "No",
                        "Has Sensitive Credentials": "No",
                        "Hostname": "ip-2-2-2-2",
                        "Image Description": "Canonical, Ubuntu, 20.04, amd64 focal image",
                        "Image ID": "ami-0123456789abcdef",
                        "Image Is Public": "True",
                        "Image Name": "ubuntu/images/hvm-ssd/ubuntu-focal-20.04-amd64-server-20250218.1",
                        "Image Owner ID": "099720109477",
                        "Ingress Ports": ["20-22"],
                        "Instance Type": "t2.micro",
                        "Is Exposed (Public/Internet Facing)": "Yes",
                        "Is Image Public": "Yes",
                        "Kernel Version": "5.15.0-1077-aws",
                        "Last Seen": "2025-03-13T04:05:49+00:00",
                        "Last Update Time": "2025-01-19T22:00:00+00:00",
                        "MAC Addresses": ["A1:B2:C3:D4:E5:F6"],
                        "Matched CVEs": 7,
                        "Max CVSS Score": 8.6,
                        "Memory": 1023,
                        "Model Tags": ["Category: Compute"],
                        "Name": "my_ec2",
                        "OS Bit Mode": 64,
                        "OS End of Support": "2025-05-31",
                        "OS Support Info Site": "https://wiki.ubuntu.com/Releases",
                        "Observations": ["internet_facing"],
                        "Private DNS": ["ip-2-2-2-2.ap-southeast-2.compute.internal"],
                        "Private IP Addresses": ["2.2.2.2"],
                        "Public IP Addresses": ["1.1.1.1"],
                        "Region": "ap-southeast-2",
                        "Regions": ["ap-southeast-2"],
                        "Regions Names": ["Sydney"],
                        "SSH Configuration Files": ["/etc/ssh/sshd_config"],
                        "State": "running",
                        "Sub Category": "VM",
                        "SubCategory (New)": "Virtual Instances",
                        "Subnet List": ["my_subnet"],
                        "Tags": ["Name: my_ec2"],
                        "Total Disks Bytes": 8589934592,
                        "Total Vulnerabilities Found": 1641,
                        "UI Unique Field": "i-abcdef01234567890",
                        "Up Time": "2025-03-12T13:55:39+00:00",
                        "Up Time Hours": 15,
                        "Updated Time": "2025-01-19T22:00:00+00:00",
                        "Vpc List": ["my_vpc"],
                        "Zones": ["ap-southeast-2b"],
                        "vCPU Count": 1,
                    },
                ],
                "group_name": "my_ec2",
                "group_type": "vm",
                "group_type_string": "NonGroup",
                "group_unique_id": "vm_111111111111_i-abcdef01234567890",
                "group_val": "nongroup",
                "is_compliance": False,
                "is_rule": True,
                "level": 0,
                "organization_id": "5f1c59f5-7c6c-4a56-a6ff-94998abdf9c3",
                "organization_name": "Partners",
                "recommendation": 'It is recommended to disable SSH password authentication by editing the SSH configuration file (/etc/ssh/sshd_config) from this line "PasswordAuthentication yes" to "PasswordAuthentication no" and enable key-pair authentication to secure SSH authentication process',
                "rule_id": "r0254751929",
                "rule_query": "Vm with Content.SSHConfigurationFiles.PasswordAuthentication and not Content.SSHConfigurationFiles.AuthenticationMethod like 'publickey'",
                "severity_contributing_factors": ["The resource is publicly exposed to the internet"],
                "source": "my_ec2",
                "state": {
                    "alert_id": "orca-123456",
                    "created_at": "2025-03-13 04:05:33.000000000",
                    "high_since": "2025-03-13 05:21:03.000000000",
                    "in_verification": False,
                    "last_seen": "2025-03-13 04:05:33.000000000",
                    "last_updated": "2025-03-13 04:05:33.000000000",
                    "orca_score": 6.2,
                    "risk_level": "medium",
                    "rule_source": "Orca",
                    "score": 3,
                    "severity": "hazardous",
                    "status": "open",
                    "status_time": "2025-03-13 04:05:33.000000000",
                },
                "subject_type": "AwsEc2Instance",
                "tags_info_list": ["Name|my_ec2"],
                "type": "ssh_user_password_authentication",
                "type_key": "9ae60ee24bb3c9a757266519efd186bf",
                "type_string": "SSH Authentication with username and password",
                "vm_asset_unique_id": "vm_111111111111_i-abcdef01234567890",
                "vm_id": "i-abcdef01234567890",
            },
        ),
        RuleTest(
            name="K8 - Vulnerable Software",
            expected_result=True,
            log={
                "p_any_ip_addresses": ["3.3.3.3", "4.4.4.4"],
                "p_event_time": "2025-03-13 02:56:19.000000000",
                "p_log_type": "Orca.Alert",
                "p_parse_time": "2025-03-13 04:55:02.265883391",
                "p_row_id": "fead7fb4e8cffece94f58ca525b523",
                "p_schema_version": 0,
                "p_source_id": "d0904c6a-e437-4082-8807-6bdccc0fc16f",
                "p_source_label": "Orca Prod",
                "p_udm": {},
                "account_name": "kubernetes-test-123456",
                "alert_labels": [
                    "authentication_bypass",
                    "easy_exploitation",
                    "fix_available",
                    "mitre: initial access",
                ],
                "alert_source": "Orca Scan",
                "asset_availability_zones": ["us-central1-c"],
                "asset_category": "VM",
                "asset_distribution_major_version": "117",
                "asset_distribution_name": "Google COS",
                "asset_distribution_version": "117-18613.164.38",
                "asset_first_private_ips": ["3.3.3.3"],
                "asset_first_public_ips": ["4.4.4.4"],
                "asset_image_id": "1234567890123456789",
                "asset_ingress_ports": ["22", "3389"],
                "asset_labels": ["public_facing"],
                "asset_name": "e2e-test",
                "asset_num_private_ips": 1,
                "asset_num_public_ips": 1,
                "asset_regions": ["us-central1"],
                "asset_regions_names": ["Council Bluffs"],
                "asset_state": "running",
                "asset_tags_info_list": [
                    "goog-gke-cluster-id-base32|averylongidnameidontknowhowlongthough",
                    "goog-k8s-cluster-location|us-central1-c",
                    "goog-k8s-cluster-name|e2e-test",
                    "goog-k8s-node-pool-name|pool-5",
                ],
                "asset_type": "gke",
                "asset_type_string": "VM",
                "asset_unique_id": "gke_kubernetes-test-123456_38e812aa286e43b6a5855faeef89beedf3b8ddc8693540eeb102809e4b403056",
                "asset_vpcs": [
                    "kubernetes-test-123456/global/networks/default",
                    "kubernetes-test-123456/regions/us-central1/subnetworks/default",
                ],
                "category": "Vulnerabilities",
                "cloud_account_id": "5f1c59f5-7c6c-4a56-a6ff-94998abdf9c3",
                "cloud_account_type": "Regular",
                "cloud_provider": "gcp",
                "cloud_provider_id": "kubernetes-test-123456",
                "cloud_vendor_id": "kubernetes-test-123456",
                "cluster_name": "e2e-test",
                "cluster_type": "gke",
                "cluster_unique_id": "gke_kubernetes-test-123456_38e812aa286e43b6a5855faeef89beedf3b8ddc8693540eeb102809e4b403056",
                "configuration": {},
                "container_id": "main",
                "context": "data",
                "cve_list": ["CVE-2024-45337"],
                "description": "The following vulnerabilities were found on software: golang.org/x/crypto-0.28.0 0.28.0",
                "details": "We have found vulnerabilities on software: golang.org/x/crypto-0.28.0 0.28.0",
                "earliest_cve_detection": "2025-03-13T00:00:00+00:00",
                "group_name": "e2e-test",
                "group_type": "gke",
                "group_type_string": "VM",
                "group_unique_id": "gke_kubernetes-test-123456_38e812aa286e43b6a5855faeef89beedf3b8ddc8693540eeb102809e4b403056",
                "group_val": "group",
                "is_compliance": False,
                "level": 0,
                "max_cvss_score": 8.2,
                "organization_id": "5f1c59f5-7c6c-4a56-a6ff-94998abdf9c3",
                "organization_name": "Partners",
                "recommendation": "Patch the listed packages",
                "rule_id": "r81a3c225f3",
                "source": "golang.org/x/crypto-0.28.0",
                "state": {
                    "alert_id": "orca-852051",
                    "created_at": "2025-03-13 02:56:19.000000000",
                    "in_verification": False,
                    "last_seen": "2025-03-13 02:56:19.000000000",
                    "last_updated": "2025-03-13 02:56:19.000000000",
                    "low_since": "2025-03-13 04:25:33.000000000",
                    "orca_score": 4.6,
                    "risk_level": "low",
                    "rule_source": "Orca",
                    "score": 4,
                    "severity": "informational",
                    "status": "open",
                    "status_time": "2025-03-13 02:56:19.000000000",
                },
                "subject_type": "vm_kubernetes-test-123456_1234567890123456789",
                "tags_info_list": [
                    "goog-gke-cluster-id-base32|averylongidnameidontknowhowlongthough",
                    "goog-k8s-cluster-location|us-central1-c",
                    "goog-k8s-cluster-name|e2e-test",
                    "goog-k8s-node-pool-name|pool-5",
                ],
                "type": "vulnerability",
                "type_key": "golang.org/x/crypto-0.28.0",
                "type_string": "Vulnerable Software",
                "vm_id": "1234567890123456789",
            },
        ),
    ]



from pypanther.rules.onepassword.onepassword_lut_sensitive_item_access import (
    OnePasswordLutSensitiveItem as OnePasswordLutSensitiveItem,
)
from pypanther.rules.onepassword.onepassword_sensitive_item_access import (
    OnePasswordSensitiveItem as OnePasswordSensitiveItem,
)
from pypanther.rules.onepassword.onepassword_unusual_client import OnePasswordUnusualClient as OnePasswordUnusualClient



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.snowflake import query_history_alert_context


@panther_managed
class SnowflakeStreamUserEnabled(Rule):
    id = "Snowflake.Stream.UserEnabled-prototype"
    display_name = "Snowflake User Enabled"
    log_types = [LogType.SNOWFLAKE_QUERY_HISTORY]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    default_description = "Detects users being re-enabled in your environment."
    tags = ["Snowflake", "[MITRE] Persistence", "[MITRE] Create Account"]
    USER_ENABLED_EXPR = re.compile("alter\\s+user\\s+(.+?)\\s+.*?set\\s+disabled\\s*=\\s*false", flags=re.I)
    USER_ENABLED = ""

    def rule(self, event):
        self.USER_ENABLED = self.USER_ENABLED_EXPR.match(event.get("QUERY_TEXT", ""))
        # Exit out early to avoid needless regex
        return all(
            (
                event.get("QUERY_TYPE") == "ALTER_USER",
                event.get("EXECUTION_STATUS") == "SUCCESS",
                self.USER_ENABLED is not None,
            ),
        )

    def title(self, event):
        enabled_user = self.USER_ENABLED.group(1)
        actor = event.get("USER_NAME", "<UNKNOWN ACTOR>")
        source = event.get("p_source_label", "<UNKNOWN SOURCE>")
        return f"{source}: Snowflake user {enabled_user} enabled by {actor}"

    def alert_context(self, event):
        return query_history_alert_context(event)

    tests = [
        RuleTest(
            name="User Enabled",
            expected_result=True,
            log={
                "p_event_time": "2024-10-09 21:03:25.750000000",
                "p_log_type": "Snowflake.QueryHistory",
                "p_row_id": "6283439ab35193e891ac9ea1227b",
                "p_source_label": "SF-Ben",
                "EXECUTION_STATUS": "SUCCESS",
                "QUERY_TEXT": "ALTER USER CLARK_KENT SET DISABLED=FALSE;",
                "QUERY_TYPE": "ALTER_USER",
                "ROLE_NAME": "ACCOUNTADMIN",
                "USER_NAME": "LEX_LUTHOR",
                "WAREHOUSE_NAME": "DATAOPS_WH",
            },
        ),
        RuleTest(
            name="User Disabled",
            expected_result=False,
            log={
                "p_event_time": "2024-10-09 21:03:25.750000000",
                "p_log_type": "Snowflake.QueryHistory",
                "p_row_id": "6283439ab35193e891ac9ea1227b",
                "p_source_label": "SF-Ben",
                "EXECUTION_STATUS": "SUCCESS",
                "QUERY_TEXT": "ALTER USER CLARK_KENT SET DISABLED=TRUE;",
                "QUERY_TYPE": "ALTER_USER",
                "ROLE_NAME": "ACCOUNTADMIN",
                "USER_NAME": "PERRY_WHITE",
                "WAREHOUSE_NAME": "DATAOPS_WH",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class AWSConsoleSignIn(Rule):
    id = "AWS.Console.Sign-In-prototype"
    display_name = "SIGNAL - AWS Console SSO Sign-In"
    create_alert = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO

    def rule(self, event):
        return event.get("eventSource") == "sso.amazonaws.com" and event.get("eventName") == "Authenticate"

    tests = [
        RuleTest(
            name="Test-94439c",
            expected_result=True,
            log={
                "awsRegion": "us-east-1",
                "eventCategory": "Management",
                "eventID": "8cb05708-9764-4774-a048-59a4c8e1684d",
                "eventName": "Authenticate",
                "eventSource": "sso.amazonaws.com",
                "eventTime": "2024-06-03 15:23:22.000000000",
                "eventType": "AwsServiceEvent",
                "eventVersion": "1.08",
                "managementEvent": True,
            },
        ),
    ]



from pypanther.rules.slack.slack_app_access_expanded import (
    SlackAuditLogsAppAccessExpanded as SlackAuditLogsAppAccessExpanded,
)
from pypanther.rules.slack.slack_app_added import SlackAuditLogsAppAdded as SlackAuditLogsAppAdded
from pypanther.rules.slack.slack_app_removed import SlackAuditLogsAppRemoved as SlackAuditLogsAppRemoved
from pypanther.rules.slack.slack_application_dos import SlackAuditLogsApplicationDoS as SlackAuditLogsApplicationDoS
from pypanther.rules.slack.slack_dlp_modified import SlackAuditLogsDLPModified as SlackAuditLogsDLPModified
from pypanther.rules.slack.slack_ekm_config_changed import (
    SlackAuditLogsEKMConfigChanged as SlackAuditLogsEKMConfigChanged,
)
from pypanther.rules.slack.slack_ekm_slackbot_unenrolled import (
    SlackAuditLogsEKMSlackbotUnenrolled as SlackAuditLogsEKMSlackbotUnenrolled,
)
from pypanther.rules.slack.slack_ekm_unenrolled import SlackAuditLogsEKMUnenrolled as SlackAuditLogsEKMUnenrolled
from pypanther.rules.slack.slack_idp_configuration_change import (
    SlackAuditLogsIDPConfigurationChanged as SlackAuditLogsIDPConfigurationChanged,
)
from pypanther.rules.slack.slack_information_barrier_modified import (
    SlackAuditLogsInformationBarrierModified as SlackAuditLogsInformationBarrierModified,
)
from pypanther.rules.slack.slack_intune_mdm_disabled import (
    SlackAuditLogsIntuneMDMDisabled as SlackAuditLogsIntuneMDMDisabled,
)
from pypanther.rules.slack.slack_legal_hold_policy_modified import (
    SlackAuditLogsLegalHoldPolicyModified as SlackAuditLogsLegalHoldPolicyModified,
)
from pypanther.rules.slack.slack_mfa_settings_changed import (
    SlackAuditLogsMFASettingsChanged as SlackAuditLogsMFASettingsChanged,
)
from pypanther.rules.slack.slack_org_created import SlackAuditLogsOrgCreated as SlackAuditLogsOrgCreated
from pypanther.rules.slack.slack_org_deleted import SlackAuditLogsOrgDeleted as SlackAuditLogsOrgDeleted
from pypanther.rules.slack.slack_passthrough_anomaly import (
    SlackAuditLogsPassthroughAnomaly as SlackAuditLogsPassthroughAnomaly,
)
from pypanther.rules.slack.slack_potentially_malicious_file_shared import (
    SlackAuditLogsPotentiallyMaliciousFileShared as SlackAuditLogsPotentiallyMaliciousFileShared,
)
from pypanther.rules.slack.slack_private_channel_made_public import (
    SlackAuditLogsPrivateChannelMadePublic as SlackAuditLogsPrivateChannelMadePublic,
)
from pypanther.rules.slack.slack_privilege_changed_to_user import (
    SlackAuditLogsUserPrivilegeChangedToUser as SlackAuditLogsUserPrivilegeChangedToUser,
)
from pypanther.rules.slack.slack_service_owner_transferred import (
    SlackAuditLogsServiceOwnerTransferred as SlackAuditLogsServiceOwnerTransferred,
)
from pypanther.rules.slack.slack_sso_settings_changed import (
    SlackAuditLogsSSOSettingsChanged as SlackAuditLogsSSOSettingsChanged,
)
from pypanther.rules.slack.slack_user_privilege_escalation import (
    SlackAuditLogsUserPrivilegeEscalation as SlackAuditLogsUserPrivilegeEscalation,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.github import github_alert_context


@panther_managed
class GitHubRepoRulesetModified(Rule):
    id = "GitHub.Repo.RulesetModified-prototype"
    display_name = "GitHub Repository Ruleset Modified"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Defense Evasion", "Impair Defenses", "Disable or Modify Tools"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_reference = "https://docs.github.com/en/repositories/configuring-branches-and-merges-in-your-repository/managing-rulesets/about-rulesets"
    default_severity = Severity.INFO
    default_description = "Disabling repository ruleset controls could indicate malicious use of admin credentials in an attempt to hide activity."
    default_runbook = "Verify that ruleset modifications are intended and authorized."

    def rule(self, event):
        return event.get("action").startswith("repository_ruleset.")

    def title(self, event):
        action = "modified"
        if event.get("action").endswith("destroy"):
            action = "deleted"
        elif event.get("action").endswith("create"):
            action = "created"
        title_str = f"Github repository ruleset for [{event.get('repo', '<UNKNOWN_REPO>')}] {action} by [{event.get('actor', '<UNKNOWN_ACTOR>')}]"
        if event.get("ruleset_source_type", default="<UNKNOWN_SOURCE_TYPE>") == "Organization":
            title_str = f"Github repository ruleset for Organization [{event.get('org', '<UNKNOWN_ORG>')}] {action} by [{event.get('actor', '<UNKNOWN_ACTOR>')}]"
        return title_str

    def dedup(self, event):
        return event.get("_document_id", "")

    def severity(self, event):
        if event.get("action").endswith("create"):
            return "INFO"
        if event.get("action").endswith("update"):
            return "MEDIUM"
        if event.get("action").endswith("destroy"):
            return "HIGH"
        return "DEFAULT"

    def alert_context(self, event):
        ctx = github_alert_context(event)
        ctx["user"] = event.get("actor", "")
        ctx["actor_is_bot"] = event.get("actor_is_bot", "")
        ctx["actor_user_agent"] = event.get("user_agent", "")
        ctx["business"] = event.get("business", "")
        ctx["public_repo"] = event.get("public_repo", "")
        ctx["operation_type"] = event.get("operation_type", "")
        ctx["ruleset_bypass_actors"] = event.deep_walk("ruleset_bypass_actors")
        ctx["ruleset_conditions"] = event.deep_walk("ruleset_conditions")
        ctx["ruleset_rules"] = event.deep_walk("ruleset_rules")
        return ctx

    tests = [
        RuleTest(
            name="GitHub - Ruleset Created",
            expected_result=True,
            log={
                "action": "repository_ruleset.create",
                "actor": "dog",
                "actor_id": "999999999",
                "actor_is_bot": False,
                "actor_location": {"country_code": "US"},
                "business": "bizname",
                "business_id": "12345",
                "created_at": "2024-12-17 00:00:00000000",
                "operation_type": "create",
                "org": "some-org",
                "org_id": 12345678,
                "public_repo": True,
                "repo": "some-org/ruleset-repo",
                "repo_id": 123456789,
                "ruleset_bypass_actors": [
                    {"actor_id": 123456, "actor_type": "Integration", "bypass_mode": "always", "id": 123456},
                    {"actor_id": 123456, "actor_type": "Team", "bypass_mode": "always", "id": 1234567},
                ],
                "ruleset_conditions": [
                    {
                        "id": 1234567,
                        "parameters": {"exclude": [], "include": ["~DEFAULT_BRANCH"]},
                        "target": "ref_name",
                    },
                ],
                "ruleset_enforcement": "enabled",
                "ruleset_id": "1234567",
                "ruleset_name": "a-ruleset-name",
                "ruleset_rules": [
                    {
                        "id": 12345678,
                        "parameters": {
                            "allowed_merge_methods": ["merge", "squash", "rebase"],
                            "authorized_dismissal_actors_only": False,
                            "automatic_copilot_code_review_enabled": False,
                            "dismiss_stale_reviews_on_push": False,
                            "ignore_approvals_from_contributors": False,
                            "require_code_owner_review": False,
                            "require_last_push_approval": False,
                            "required_approving_review_count": 1,
                            "required_review_thread_resolution": False,
                            "required_reviewers": [],
                        },
                        "type": "pull_request",
                    },
                    {"id": 12345678, "parameters": {}, "type": "deletion"},
                    {"id": 12345678, "parameters": {}, "type": "non_fast_forward"},
                ],
                "ruleset_source_type": "Repository",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
            },
        ),
        RuleTest(
            name="GitHub - Ruleset Deleted",
            expected_result=True,
            log={
                "action": "repository_ruleset.destroy",
                "actor": "dog",
                "actor_id": "999999999",
                "actor_is_bot": False,
                "actor_location": {"country_code": "US"},
                "business": "bizname",
                "business_id": "12345",
                "created_at": "2024-12-17 00:00:00000000",
                "operation_type": "remove",
                "org": "some-org",
                "org_id": 12345678,
                "public_repo": False,
                "repo": "some-org/ruleset-repo",
                "repo_id": 123456789,
                "ruleset_bypass_actors": [
                    {"actor_id": 123456, "actor_type": "Integration", "bypass_mode": "always", "id": 123456},
                    {"actor_id": 123456, "actor_type": "Team", "bypass_mode": "always", "id": 1234567},
                ],
                "ruleset_conditions": [
                    {
                        "id": 1234567,
                        "parameters": {"exclude": [], "include": ["~DEFAULT_BRANCH"]},
                        "target": "ref_name",
                    },
                ],
                "ruleset_enforcement": "enabled",
                "ruleset_id": "1234567",
                "ruleset_name": "a-ruleset-name",
                "ruleset_rules": [
                    {
                        "id": 10994218,
                        "parameters": {
                            "allowed_merge_methods": ["merge", "squash", "rebase"],
                            "authorized_dismissal_actors_only": False,
                            "automatic_copilot_code_review_enabled": True,
                            "dismiss_stale_reviews_on_push": False,
                            "ignore_approvals_from_contributors": False,
                            "require_code_owner_review": False,
                            "require_last_push_approval": False,
                            "required_approving_review_count": 1,
                            "required_review_thread_resolution": False,
                            "required_reviewers": [],
                        },
                        "type": "pull_request",
                    },
                    {"id": 10994219, "parameters": {}, "type": "deletion"},
                    {"id": 10994220, "parameters": {}, "type": "non_fast_forward"},
                ],
                "ruleset_source_type": "Repository",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
            },
        ),
        RuleTest(
            name="GitHub - Non Webhook Event",
            expected_result=False,
            log={
                "actor": "cat",
                "action": "org.invite_member",
                "org": "my-org",
                "p_log_type": "GitHub.Audit",
                "repo": "my-org/my-repo",
            },
        ),
        RuleTest(
            name="Github - Ruleset Modified",
            expected_result=True,
            log={
                "action": "repository_ruleset.update",
                "actor": "dog",
                "actor_id": "999999999",
                "actor_is_bot": False,
                "actor_location": {"country_code": "US"},
                "business": "bizname",
                "business_id": "12345",
                "created_at": "2024-12-17 00:00:00000000",
                "operation_type": "modify",
                "org": "some-org",
                "org_id": 12345678,
                "public_repo": False,
                "repo": "some-org/ruleset-repo",
                "repo_id": 123456789,
                "ruleset_bypass_actors": [
                    {"actor_id": 123456, "actor_type": "Integration", "bypass_mode": "always", "id": 123456},
                    {"actor_id": 123456, "actor_type": "Team", "bypass_mode": "always", "id": 1234567},
                ],
                "ruleset_conditions": [
                    {
                        "id": 1234567,
                        "parameters": {"exclude": [], "include": ["~DEFAULT_BRANCH"]},
                        "target": "ref_name",
                    },
                ],
                "ruleset_enforcement": "enabled",
                "ruleset_id": "1234567",
                "ruleset_name": "a-ruleset-name",
                "ruleset_rules_updated": [
                    {
                        "id": 12345678,
                        "old_parameters": {
                            "allowed_merge_methods": ["merge", "squash", "rebase"],
                            "authorized_dismissal_actors_only": False,
                            "automatic_copilot_code_review_enabled": False,
                            "dismiss_stale_reviews_on_push": False,
                            "ignore_approvals_from_contributors": False,
                            "require_code_owner_review": False,
                            "require_last_push_approval": False,
                            "required_approving_review_count": 1,
                            "required_review_thread_resolution": False,
                            "required_reviewers": [],
                        },
                        "parameters": {
                            "allowed_merge_methods": ["merge", "squash", "rebase"],
                            "authorized_dismissal_actors_only": False,
                            "automatic_copilot_code_review_enabled": True,
                            "dismiss_stale_reviews_on_push": False,
                            "ignore_approvals_from_contributors": False,
                            "require_code_owner_review": False,
                            "require_last_push_approval": False,
                            "required_approving_review_count": 1,
                            "required_review_thread_resolution": False,
                            "required_reviewers": [],
                        },
                        "type": "pull_request",
                    },
                ],
                "ruleset_source_type": "Repository",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
            },
        ),
        RuleTest(
            name="Github - Ruleset Modified for entire Organization",
            expected_result=True,
            log={
                "action": "repository_ruleset.update",
                "actor": "dog",
                "actor_id": "999999999",
                "actor_is_bot": False,
                "actor_location": {"country_code": "US"},
                "business": "bizname",
                "business_id": "12345",
                "created_at": "2024-12-17 00:00:00000000",
                "operation_type": "modify",
                "org": "some-org",
                "org_id": 12345678,
                "ruleset_enforcement": "disabled",
                "ruleset_id": "1082915",
                "ruleset_name": "Name of org-wide ruleset",
                "ruleset_old_enforcement": "evaluate",
                "ruleset_source_type": "Organization",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/132.0.0.0 Safari/537.36",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSCloudTrailCodebuildProjectMadePublic(Rule):
    id = "AWS.CloudTrail.CodebuildProjectMadePublic-prototype"
    display_name = "CodeBuild Project made Public"
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    tags = ["AWS", "Security Control", "Exfiltration:Exfiltration Over Web Service"]
    default_severity = Severity.HIGH
    default_description = "An AWS CodeBuild Project was made publicly accessible\n"
    default_runbook = "TBD"
    default_reference = "https://docs.aws.amazon.com/codebuild/latest/userguide/public-builds.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]

    def rule(self, event):
        return (
            event["eventName"] == "UpdateProjectVisibility"
            and event.deep_get("requestParameters", "projectVisibility") == "PUBLIC_READ"
        )

    def title(self, event):
        return f"AWS CodeBuild Project made Public by {event.deep_get('userIdentity', 'arn')} in account {event.deep_get('recipientAccountId')}"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="CodeBuild Project Made Public",
            expected_result=True,
            log={
                "eventVersion": "1.08",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "111111111111",
                    "arn": "arn:aws:sts::111122223333:assumed-role/MakeStuffPublic",
                    "accountId": "111122223333",
                    "accessKeyId": "ASIAXXXXXXXXXXXX",
                    "sessionContext": {
                        "sessionIssuer": {},
                        "webIdFederationData": {},
                        "attributes": {"creationDate": "2021-08-18T14:54:10Z", "mfaAuthenticated": "false"},
                    },
                },
                "eventTime": "2021-08-18T14:54:53Z",
                "eventSource": "codebuild.amazonaws.com",
                "eventName": "UpdateProjectVisibility",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "1.1.1.1",
                "userAgent": "aws-internal/3 aws-sdk-java/1.11.1030 Linux/5.4.116-64.217.amzn2int.x86_64 OpenJDK_64-Bit_Server_VM/25.302-b08 java/1.8.0_302 vendor/Oracle_Corporation cfg/retry-mode/legacy",
                "requestParameters": {
                    "projectVisibility": "PUBLIC_READ",
                    "projectArn": "arn:aws:codebuild:us-east-1:111122223333:project/testproject1234",
                    "resourceAccessRole": "arn:aws:iam::111122223333:role/service-role/test",
                },
                "responseElements": None,
                "requestID": "4397365f-c790-4c23-9fe6-97e13a16ea84",
                "eventID": "982f8066-640d-40fb-b433-ba15e14fee40",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "managementEvent": True,
                "recipientAccountId": "111122223333",
                "eventCategory": "Management",
            },
        ),
        RuleTest(
            name="CodeBuild Project Made Private",
            expected_result=False,
            log={
                "eventVersion": "1.08",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "111111111111",
                    "arn": "arn:aws:sts::111122223333:assumed-role/MakeStuffPublic",
                    "accountId": "111122223333",
                    "accessKeyId": "ASIAXXXXXXXXXXXX",
                    "sessionContext": {
                        "sessionIssuer": {},
                        "webIdFederationData": {},
                        "attributes": {"creationDate": "2021-08-18T14:54:10Z", "mfaAuthenticated": "false"},
                    },
                },
                "eventTime": "2021-08-18T14:54:53Z",
                "eventSource": "codebuild.amazonaws.com",
                "eventName": "UpdateProjectVisibility",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "1.1.1.1",
                "userAgent": "aws-internal/3 aws-sdk-java/1.11.1030 Linux/5.4.116-64.217.amzn2int.x86_64 OpenJDK_64-Bit_Server_VM/25.302-b08 java/1.8.0_302 vendor/Oracle_Corporation cfg/retry-mode/legacy",
                "requestParameters": {
                    "projectVisibility": "PRIVATE",
                    "projectArn": "arn:aws:codebuild:us-east-1:111122223333:project/testproject1234",
                    "resourceAccessRole": "arn:aws:iam::111122223333:role/service-role/test",
                },
                "responseElements": None,
                "requestID": "4397365f-c790-4c23-9fe6-97e13a16ea84",
                "eventID": "982f8066-640d-40fb-b433-ba15e14fee40",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "managementEvent": True,
                "recipientAccountId": "111122223333",
                "eventCategory": "Management",
            },
        ),
        RuleTest(
            name="Not a UpdateProjectVisibility event",
            expected_result=False,
            log={
                "eventVersion": "1.08",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "111111111111",
                    "arn": "arn:aws:sts::111122223333:assumed-role/MakeStuffPublic",
                    "accountId": "111122223333",
                    "accessKeyId": "ASIAXXXXXXXXXXXX",
                    "sessionContext": {
                        "sessionIssuer": {},
                        "webIdFederationData": {},
                        "attributes": {"creationDate": "2021-08-18T14:54:10Z", "mfaAuthenticated": "false"},
                    },
                },
                "eventTime": "2021-08-18T14:54:53Z",
                "eventSource": "codebuild.amazonaws.com",
                "eventName": "CreateProject",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "1.1.1.1",
                "userAgent": "aws-internal/3 aws-sdk-java/1.11.1030 Linux/5.4.116-64.217.amzn2int.x86_64 OpenJDK_64-Bit_Server_VM/25.302-b08 java/1.8.0_302 vendor/Oracle_Corporation cfg/retry-mode/legacy",
                "responseElements": None,
                "requestID": "4397365f-c790-4c23-9fe6-97e13a16ea84",
                "eventID": "982f8066-640d-40fb-b433-ba15e14fee40",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "managementEvent": True,
                "recipientAccountId": "111122223333",
                "eventCategory": "Management",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GitHubSecretScanningAlertCreated(Rule):
    id = "GitHub.Secret.Scanning.Alert.Created-prototype"
    display_name = "GitHub Secret Scanning Alert Created"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub"]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    default_severity = Severity.MEDIUM
    default_description = "GitHub detected a secret and created a secret scanning alert."
    default_runbook = "Review the secret to determine if it needs to be revoked or the alert suppressed."
    default_reference = "https://docs.github.com/en/code-security/secret-scanning/about-secret-scanning"

    def rule(self, event):
        return event.get("action", "") == "secret_scanning_alert.create"

    def title(self, event):
        return f"Github detected a secret in {event.get('repo', '<REPO_NOT_FOUND>')} (#{event.get('number', '<NUMBER_NOT_FOUND>')})"

    def alert_context(self, event):
        return {
            "github_organization": event.get("org", "<ORG_NOT_FOUND>"),
            "github_repository": event.get("repo", "<REPO_NOT_FOUND>"),
            "alert_number": str(event.get("number", "<NUMBER_NOT_FOUND>")),
            "url": f"https://github.com/{event.get('repo')}/security/secret-scanning/{event.get('number')}"
            if all([event.get("repo"), event.get("number")])
            else "<URL_NOT_FOUND>",
        }

    tests = [
        RuleTest(
            name="secret_scanning_alert.create-true",
            expected_result=True,
            log={
                "action": "secret_scanning_alert.create",
                "actor": "github",
                "actor_id": "1234",
                "business": "Acme Inc.",
                "business_id": "12345",
                "created_at": "2023-10-18 18:20:52.209000000",
                "number": 12,
                "org": "acme-inc",
                "org_id": 1234567,
                "repo": "acme-inc/crown-jewels",
                "repo_id": 123456789,
            },
        ),
        RuleTest(
            name="git.clone-false",
            expected_result=False,
            log={
                "_document_id": "KCYtigpnShPBSohA4OXbRg==",
                "action": "git.clone",
                "actor": "acme-inc-user",
                "actor_id": "123456789",
                "actor_ip": "5.6.7.8",
                "actor_location": {"country_code": "US"},
                "at_sign_timestamp": "2023-11-20 21:20:09.423",
                "business": "acme-inc",
                "business_id": "12345",
                "external_identity_nameid": "",
                "external_identity_username": "",
                "hashed_token": "0771cae4d170dc02a6ff393d9946cd684d6145c1=",
                "org": "acme-inc",
                "org_id": 12345678,
                "programmatic_access_type": "OAuth access token",
                "repo": "acme-inc/a-repo",
                "repository": "acme-inc/a-repo",
                "repository_public": False,
                "token_id": "0123456789",
                "transport_protocol": 1,
                "transport_protocol_name": "http",
                "user": "",
                "user_agent": "git/2.30.2",
                "user_id": "0",
            },
        ),
    ]



from panther_detection_helpers.caching import check_account_age

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSCloudTrailAMIModifiedForPublicAccess(Rule):
    id = "AWS.CloudTrail.AMIModifiedForPublicAccess-prototype"
    display_name = "Amazon Machine Image (AMI) Modified to Allow Public Access"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Exfiltration:Transfer Data to Cloud Account"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0010:T1537"]}
    default_description = "An Amazon Machine Image (AMI) was modified to allow it to be launched by anyone. Any sensitive configuration or application data stored in the AMI's block devices is at risk.\n"
    default_runbook = "Determine if the AMI is intended to be publicly accessible. If not, first modify the AMI to not be publicly accessible then change any sensitive data stored in the block devices associated to the AMI (as they may be compromised).\n"
    default_reference = "https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/sharingamis-intro.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]

    def rule(self, event):
        # Only check successful ModiyImageAttribute events
        if not aws_cloudtrail_success(event) or event.get("eventName") != "ModifyImageAttribute":
            return False
        added_perms = event.deep_get("requestParameters", "launchPermission", "add", "items", default=[{}])
        for item in added_perms:
            if item.get("group") == "all":
                return True
            if check_account_age(
                item.get("userId", "") + "-" + event.udm("user_account_id", default=""),
            ):  # checking if the account is new
                return True
        return False

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="AMI Made Public",
            expected_result=True,
            log={
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "ModifyImageAttribute",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "attributeType": "launchPermission",
                    "imageId": "ami-1111",
                    "launchPermission": {"add": {"items": [{"group": "all"}]}},
                },
                "responseElements": {"_return": True},
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="AMI Not Made Public",
            expected_result=False,
            mocks=[RuleMock(object_name="check_account_age", return_value=False)],
            log={
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "ModifyImageAttribute",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "attributeType": "launchPermission",
                    "imageId": "ami-1111",
                    "launchPermission": {"add": {"items": [{"group": "not-all"}]}},
                },
                "responseElements": {"_return": True},
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="AMI Launch Permissions Not Modified",
            expected_result=False,
            mocks=[RuleMock(object_name="check_account_age", return_value=False)],
            log={
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "ModifyImageAttribute",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "attributeType": "someThing",
                    "imageId": "ami-1111",
                    "someThing": {"add": {"items": [{"group": "all"}]}},
                },
                "responseElements": {"_return": True},
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="AMI Added to User",
            expected_result=False,
            mocks=[RuleMock(object_name="check_account_age", return_value=False)],
            log={
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "ModifyImageAttribute",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "attributeType": "launchPermission",
                    "imageId": "ami-1111",
                    "launchPermission": {"add": {"items": [{"userId": "bob"}]}},
                },
                "responseElements": {"_return": True},
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Error Making AMI Public",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "errorCode": "UnauthorizedOperation",
                "eventID": "1111",
                "eventName": "ModifyImageAttribute",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "attributeType": "launchPermission",
                    "imageId": "ami-1111",
                    "launchPermission": {"add": {"items": [{"group": "all"}]}},
                },
                "responseElements": {"_return": True},
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Access Granted To Unknown User",
            expected_result=True,
            mocks=[RuleMock(object_name="check_account_age", return_value=True)],
            log={
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "ModifyImageAttribute",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "attributeType": "launchPermission",
                    "imageId": "ami-1111",
                    "launchPermission": {"add": {"items": [{"userId": "012345678901"}]}},
                },
                "responseElements": {"_return": True},
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class DefenderDetectionpassthrough(Rule):
    id = "Defender.Detection.passthrough-prototype"
    display_name = "Defender Detection Passthrough"
    log_types = [LogType.MICROSOFT_DEFENDER_X_D_R_ADVANCED_HUNTING]
    tags = ["Defender"]
    default_severity = Severity.MEDIUM
    default_description = "Microsoft Defender has detected malicious activity. This activty could be on a host or on a connected platform such as Azure, Microsoft 365, or Intune."
    default_runbook = "."
    default_reference = "https://learn.microsoft.com/en-us/defender-xdr/advanced-hunting-alertinfo-table"
    ALERT_URL = ""

    def rule(self, event):
        # Alert on any AlertInfo event
        return event.get("category") == "AdvancedHunting-AlertInfo"

    def title(self, event):
        # Simple title with the native Defender alert title
        return f"Defender Alert: [{event.deep_get('properties', 'Title', default='Unknown')}]"

    def alert_context(self, event):
        # Use the AlertId and tenantId to generate a URL to the alert in the Microsoft 365 Security
        # Center. The tenant ID is not completely necessary, but is helpful if a user is a member
        # of multiple tenants
        alert_id = event.deep_get("properties", "AlertId", default="Unknown")
        tenant_id = event.deep_get("tenantId", default="Unknown")
        if alert_id != "Unknown" and tenant_id != "Unknown":
            self.ALERT_URL = f"https://security.microsoft.com/alerts/{alert_id}?tid={tenant_id}"
        else:
            self.ALERT_URL = ""
        return {
            "AlertId": alert_id,
            "Name": event.deep_get("properties", "Title", default="Unknown"),
            "Severity": event.deep_get("properties", "Severity", default="Unknown"),
            "Source": event.deep_get("properties", "DetectionSource", default="Unknown"),
            "ATT&CK Techniques": event.deep_get("properties", "AttackTechniques", default="Unknown"),
            "Alert URL": self.ALERT_URL,
        }

    def reference(self, _):
        # If the alert ID is not found, return the Microsoft Defender documentation for the
        # AlertInfo table.
        if self.ALERT_URL:
            return self.ALERT_URL
        return "https://learn.microsoft.com/en-us/defender-xdr/advanced-hunting-alertinfo-table"

    def severity(self, event):
        return event.deep_get("properties", "Severity", default="MEDIUM")

    def dedup(self, event):
        return f"{event.deep_get('properties', 'AlertId', default='Unknown')} "

    tests = [
        RuleTest(
            name="High Severity Finding",
            expected_result=True,
            log={
                "_TimeReceivedBySvc": "2025-04-11 21:07:42.726000000",
                "category": "AdvancedHunting-AlertInfo",
                "operationName": "Publish",
                "properties": {
                    "AlertId": "0000000000-1111-2222-3333-444444444444",
                    "AttackTechniques": '["System Binary Proxy Execution (T1218)","Regsvr32 (T1218.010)","Rundll32 (T1218.011)"]',
                    "Category": "DefenseEvasion",
                    "DetectionSource": "Scheduled Alerts",
                    "ServiceSource": "Microsoft Sentinel",
                    "Severity": "High",
                    "Timestamp": "2025-04-21 20:51:04.058183200",
                    "Title": "Regsvr32 Rundll32 Image Loads Abnormal Extension",
                },
                "Tenant": "DefaultTenant",
                "tenantId": "55555555-6666-7777-8888-999999999999",
                "time": "2025-04-11 21:07:42.781656600",
            },
        ),
        RuleTest(
            name="Alert Evidence Event",
            expected_result=False,
            log={
                "_TimeReceivedBySvc": "2025-04-11 21:07:42.759000000",
                "category": "AdvancedHunting-AlertEvidence",
                "operationName": "Publish",
                "properties": {
                    "AdditionalFields": '{"DnsDomain":"test.com","HostName":"testdevice","NetBiosName":"testdevice.testdomain.com","IsDomainJoined":true,"RbacScopes":{"ScopesPerType":{"Workloads":{"Mode":"All","Scopes":["Sentinel"]},"Workspace":{"Mode":"All","Scopes":["82f5e56e-5ba9-421a-9556-03a95490c2a7"]}}},"Type":"host","MachineId":"0123456789abcdef0123456789abcdef","MachineIdType":3,"Role":0,"MergeByKey":"x/NzTINVL5AfAzJZ5aSiYJOW9uQ=","MergeByKeyHex":"C7F3734C83552F901F033259E5A4A2609396F6E4"}',
                    "AlertId": "sn34c87846-505a-4c84-b125-44bce1202df4",
                    "AttackTechniques": '["System Binary Proxy Execution (T1218)","Regsvr32 (T1218.010)","Rundll32 (T1218.011)"]',
                    "Categories": '["DefenseEvasion"]',
                    "CloudPlatform": "",
                    "DetectionSource": "Scheduled Alerts",
                    "DeviceId": "0123456789abcdef0123456789abcdef",
                    "DeviceName": "testdevice.testdomain.com",
                    "EntityType": "Machine",
                    "EvidenceRole": "Impacted",
                    "ServiceSource": "Microsoft Sentinel",
                    "Severity": "High",
                    "SubscriptionId": "",
                    "Timestamp": "2025-04-11 20:51:04.058183200",
                    "Title": "Regsvr32 Rundll32 Image Loads Abnormal Extension",
                },
                "Tenant": "DefaultTenant",
                "tenantId": "2008f78a-ebba-4182-8f80-7d9a0aec27a5",
                "time": "2025-04-11 21:07:42.781258000",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPIAMTagEnumeration(Rule):
    id = "GCP.IAM.Tag.Enumeration-prototype"
    default_description = "Detects enumeration of IAM policies and tags in GCP, which could be a precursor to privilege escalation attempts via tag-based access control.\n"
    display_name = "GCP IAM and Tag Enumeration"
    log_types = [LogType.GCP_AUDIT_LOG]
    create_alert = False
    default_runbook = "Review if the user has legitimate business need for these enumeration operations. If unauthorized, review and update IAM policies.\n"
    default_severity = Severity.INFO
    tags = ["attack.reconnaissance", "attack.t1548", "gcp", "iam", "tagbinding"]

    def rule(self, event):
        enum_iam_tags = [
            "GetIamPolicy",
            "TagKeys.ListTagKeys",
            "TagKeys.ListTagValues",
            "TagBindings.ListEffectiveTags",
        ]
        method_name = event.deep_get("protoPayload", "methodName", default="")
        return any(tag in method_name for tag in enum_iam_tags)

    def title(self, event):
        principal = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<UNKNOWN>")
        method = event.deep_get("protoPayload", "methodName", default="<UNKNOWN>")
        return f"GCP IAM and Tag Enumeration by {principal} - {method}"

    def alert_context(self, event):
        return gcp_alert_context(event)

    tests = [
        RuleTest(
            name="IAM Policy Enumeration",
            expected_result=True,
            log={
                "protoPayload": {
                    "methodName": "GetIamPolicy",
                    "authenticationInfo": {"principalEmail": "test@example.com"},
                    "resourceName": "projects/test-project",
                },
                "resource": {"labels": {"project_id": "test-project"}},
                "timestamp": "2024-01-01T00:00:00Z",
            },
        ),
        RuleTest(
            name="Tag Keys Enumeration",
            expected_result=True,
            log={
                "protoPayload": {
                    "methodName": "TagKeys.ListTagKeys",
                    "authenticationInfo": {"principalEmail": "test@example.com"},
                    "resourceName": "projects/test-project",
                },
                "resource": {"labels": {"project_id": "test-project"}},
                "timestamp": "2024-01-01T00:00:00Z",
            },
        ),
        RuleTest(
            name="Normal Operation",
            expected_result=False,
            log={
                "protoPayload": {
                    "methodName": "compute.instances.list",
                    "authenticationInfo": {"principalEmail": "test@example.com"},
                    "resourceName": "projects/test-project",
                },
                "resource": {"labels": {"project_id": "test-project"}},
                "timestamp": "2024-01-01T00:00:00Z",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.github import github_alert_context


@panther_managed
class GitHubWebhookModified(Rule):
    id = "GitHub.Webhook.Modified-prototype"
    display_name = "GitHub Web Hook Modified"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Exfiltration:Automated Exfiltration"]
    reports = {"MITRE ATT&CK": ["TA0010:T1020"]}
    default_reference = "https://docs.github.com/en/webhooks/about-webhooks"
    default_severity = Severity.INFO
    default_description = "Detects when a webhook is added, modified, or deleted"

    def rule(self, event):
        return event.get("action").startswith("hook.")

    def title(self, event):
        repo = event.get("repo", "<UNKNOWN_REPO>")
        action = "modified"
        if event.get("action").endswith("destroy"):
            action = "deleted"
        elif event.get("action").endswith("create"):
            action = "created"
        title_str = f"Github webhook [{event.deep_get('config', 'url', default='<UNKNOWN_URL>')}] {action} by [{event.get('actor', '<UNKNOWN_ACTOR>')}]"
        if repo != "<UNKNOWN_REPO>":
            title_str += f" in repository [{repo}]"
        return title_str

    def severity(self, event):
        if event.get("action").endswith("create"):
            return "MEDIUM"
        return "INFO"

    def alert_context(self, event):
        ctx = github_alert_context(event)
        ctx["business"] = event.get("business", "")
        ctx["hook_id"] = event.get("hook_id", "")
        ctx["integration"] = event.get("integration", "")
        ctx["operation_type"] = event.get("operation_type", "")
        ctx["url"] = event.deep_get("config", "url", default="<UNKNOWN_URL>")
        return ctx

    tests = [
        RuleTest(
            name="GitHub - Webhook Created",
            expected_result=True,
            log={
                "actor": "cat",
                "action": "hook.create",
                "data": {
                    "hook_id": 111222333444555,
                    "events": ["fork", "public", "pull_request", "push", "repository"],
                },
                "config": {"url": "https://fake.url"},
                "org": "my-org",
                "p_log_type": "GitHub.Audit",
                "repo": "my-org/my-repo",
                "public_repo": False,
            },
        ),
        RuleTest(
            name="GitHub - Webhook Deleted",
            expected_result=True,
            log={
                "actor": "cat",
                "action": "hook.destroy",
                "data": {
                    "hook_id": 111222333444555,
                    "events": ["fork", "public", "pull_request", "push", "repository"],
                },
                "org": "my-org",
                "p_log_type": "GitHub.Audit",
                "repo": "my-org/my-repo",
                "public_repo": False,
            },
        ),
        RuleTest(
            name="GitHub - Non Webhook Event",
            expected_result=False,
            log={
                "actor": "cat",
                "action": "org.invite_member",
                "org": "my-org",
                "p_log_type": "GitHub.Audit",
                "repo": "my-org/my-repo",
            },
        ),
        RuleTest(
            name="Github - App Webhook Created",
            expected_result=True,
            log={
                "action": "hook.create",
                "actor": "dog",
                "actor_id": "11112222",
                "actor_location": {"country_code": "US"},
                "business": "my-biz",
                "business_id": "9999999",
                "config": {"content_type": "json", "insecure_ssl": "0", "url": "https://fake.url/"},
                "hook_id": "111222333444555",
                "integration": "My Cool Github Integration",
                "name": "webhook",
                "operation_type": "create",
                "org": "my-org",
                "org_id": 9999999,
                "p_log_type": "GitHub.Audit",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.zscaler import zia_alert_context, zia_success


@panther_managed
class ZIABackupDeleted(Rule):
    id = "ZIA.Backup.Deleted-prototype"
    default_description = "This rule detects when ZIA backup data was deleted."
    display_name = "ZIA Backup Deleted"
    default_runbook = "Verify that this change was planned. If not, make sure to restore the backup."
    default_reference = "https://help.zscaler.com/zia/about-backup-and-restore"
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0005:T1562.008"]}
    log_types = [LogType.ZSCALER_ZIA_ADMIN_AUDIT_LOG]

    def rule(self, event):
        if not zia_success(event):
            return False
        action = event.deep_get("event", "action", default="ACTION_NOT_FOUND")
        category = event.deep_get("event", "category", default="CATEGORY_NOT_FOUND")
        if action == "DELETE" and category == "BACKUP_AND_RESTORE":
            return True
        return False

    def title(self, event):
        return f"[Zscaler.ZIA]: Backup was deleted by admin with id [{event.deep_get('event', 'adminid', default='<ADMIN_ID_NOT_FOUND>')}]"

    def alert_context(self, event):
        return zia_alert_context(event)

    tests = [
        RuleTest(
            name="Backup deleted",
            expected_result=True,
            log={
                "event": {
                    "action": "DELETE",
                    "adminid": "admin@test.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "BACKUP_AND_RESTORE",
                    "clientip": "1.2.3.4",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {},
                    "preaction": {
                        "adminLogin": "admin@test.zscalerbeta.net",
                        "goldenRestorePoint": False,
                        "id": 163372,
                        "name": "test-restore-2",
                        "time": 1730737925000,
                    },
                    "recordid": "366",
                    "resource": "test-restore-2",
                    "result": "SUCCESS",
                    "subcategory": "BACKUP_AND_RESTORE",
                    "time": "2024-11-04 16:32:18.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
        RuleTest(
            name="Backup created",
            expected_result=False,
            log={
                "event": {
                    "action": "CREATE",
                    "adminid": "admin@test.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "BACKUP_AND_RESTORE",
                    "clientip": "1.2.3.4",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {
                        "adminLogin": "admin@test.zscalerbeta.net",
                        "goldenRestorePoint": False,
                        "id": 163372,
                        "name": "test-restore-2",
                        "time": 1730737925000,
                    },
                    "preaction": {"goldenRestorePoint": False, "id": 0, "name": "test-restore-2", "time": 0},
                    "recordid": "365",
                    "resource": "test-restore-2",
                    "result": "SUCCESS",
                    "subcategory": "BACKUP_AND_RESTORE",
                    "time": "2024-11-04 16:32:05.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
    ]



from panther_core import PantherEvent

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPComputeSSHConnection(Rule):
    id = "GCP.Compute.SSHConnection-prototype"
    display_name = "GCP Compute SSH Connection"
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.INFO
    default_description = "Detect any SSH connections to a Compute Instance.\n"
    default_reference = "https://cloud.google.com/compute/docs/connect/ssh-best-practices/auditing\n"
    tags = ["GCP", "GCP.AuditLog", "SSH", "Compute", "Beta"]

    def rule(self, event: PantherEvent) -> bool:
        service_name = event.deep_get("protoPayload", "serviceName", default="")
        method_name = event.deep_get("protoPayload", "methodName", default="")
        if service_name == "iap.googleapis.com" and method_name == "AuthorizeUser":
            return True
        if service_name == "oslogin.googleapis.com":
            if any([method_name.endswith(".CheckPolicy"), method_name.endswith(".ContinueSession")]):
                return True
        if service_name == "compute.googleapis.com":
            # Check attempts to add SSH keys to the VM
            # setCommonInstanceMetadata is triggered when SSHing from a remote device
            # setMetadata is triggered when SSHing from the GCP Console
            ssh_keys = {"ssh-keys", "sshKeys"}  # Fields indicating the SSH keys were modified
            if any([method_name.endswith(".setCommonInstanceMetadata"), method_name.endswith(".setMetadata")]):
                # The metadata delta field could be for the project or the instance, and could indicate
                # something was removed or modified. We need to check all possible paths to the field
                # we need.
                modified_keys = set()
                for field1 in ["projectMetadataDelta", "instanceMetadataDelta"]:
                    for field2 in ["addedMetadataKeys", "modifiedMetadataKeys"]:
                        modified_keys.update(
                            set(event.deep_get("protoPayload", "metadata", field1, field2, default=[])),
                        )
                return bool(modified_keys & ssh_keys)
        # Check direct connections to the serial console
        # The actual service name has the region included, so we just so an easy check here
        if service_name.endswith("ssh-serialport.googleapis.com"):
            if method_name == "google.ssh-serialport.v1.connect":
                return "succeeded" in event.deep_get("protoPayload", "status", "message", default="").lower()
        return False

    def alert_context(self, event: PantherEvent) -> dict:
        instance_info = self.get_instance_info(event)
        context = {
            "instance_id": instance_info.get("id", "UNKNOWN INSTANCE ID"),
            "instance_name": instance_info.get("name", "UNKNOWN INSTANCE NAME"),
        }
        return gcp_alert_context(event) | context

    def get_instance_info(self, event: PantherEvent) -> dict:
        service_name = event.deep_get("protoPayload", "serviceName", default="")
        context = {"id": "UNKNOWN INSTANCE ID", "name": "UNKNOWN INSTANCE NAME"}
        # Name is not included in the event
        match service_name:
            case "iap.googleapis.com":
                context |= {"id": event.deep_get("resource", "labels", "instance_id", default="UNKNOWN INSTANCE ID")}
            case "oslogin.googleapis.com":
                context |= {
                    "id": event.deep_get("labels", "instance_id", default="UNKNOWN INSTANCE ID"),
                    "name": event.deep_get("protoPayload", "request", "instance", default="UNKNOWN INSTANCE NAME"),
                }
            case "compute.googleapis.com":
                if event.deep_get("protoPayload", "methodName", default="").endswith(".setCommonInstanceMetadata"):
                    # These events are targeted prokect-wide, so they don't have information about
                    # specific instances.
                    pass
                else:
                    # Will look like: projects/project-name/zones/zone-name/instances/instance-name
                    context |= {
                        "name": event.deep_get("protoPayload", "resourceName", default="/UNKNOWN INSTANCE NAME").split(
                            "/",
                        )[-1],
                    }
        if "serialport" in service_name:
            # Will look like:
            # projects/projectName/zones/zoneName/instances/instanceName/SerialPort/portNum
            context |= {
                "id": event.deep_get("resource", "labels", "instance_id", default="UNKNOWN INSTANCE ID"),
                "name": event.deep_get("protoPayload", "resourceName", default="/UNKNOWN INSTANCE NAME//").split("/")[
                    -3
                ],
            }
        return context

    tests = [
        RuleTest(
            name="Connect with IAP",
            expected_result=True,
            log={
                "p_any_ip_addresses": ["192.168.1.100"],
                "p_any_emails": ["user@example.com"],
                "p_any_usernames": ["user"],
                "p_event_time": "2025-05-27 16:46:46.485356507",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2025-05-27 19:05:21.311995228",
                "p_row_id": "fee0f92d7864a191dfa994e326d28304",
                "p_schema_version": 0,
                "p_source_id": "bd7da315-647e-4eca-bcfe-083fab18f3f1",
                "p_source_label": "gcp-logsource",
                "p_udm": {
                    "source": {"address": "192.168.1.100", "ip": "192.168.1.100"},
                    "user": {"email": "user@example.com"},
                },
                "insertId": "1rk2tche2xh0e",
                "logName": "projects/example-project/logs/cloudaudit.googleapis.com%2Fdata_access",
                "operation": {"id": "Q444-UYUD-GBRY-QFUF-AS7Q-6A6E", "producer": "iap.googleapis.com"},
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@example.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "iap.tunnelInstances.accessViaIAP",
                            "resource": "projects/123456789012/iap_tunnel/zones/us-central1-f/instances/1234567890123456789",
                            "resourceAttributes": {
                                "name": "projects/123456789012/iap_tunnel/zones/us-central1-f/instances/1234567890123456789",
                                "service": "iap.googleapis.com",
                                "type": "iap.googleapis.com/TunnelInstance",
                            },
                        },
                    ],
                    "metadata": {
                        "device_id": "",
                        "device_state": "Unknown",
                        "iap_tcp_session_info": {"bytes_received": 6922, "bytes_sent": 2874, "phase": "SESSION_END"},
                        "oauth_client_id": "",
                        "request_id": "1640143122448486764",
                    },
                    "methodName": "AuthorizeUser",
                    "request": {
                        "@type": "type.googleapis.com/cloud.security.gatekeeper.AuthorizeUserRequest",
                        "httpRequest": {"url": ""},
                    },
                    "requestMetadata": {
                        "callerIP": "192.168.1.100",
                        "callerSuppliedUserAgent": "(none supplied)",
                        "destinationAttributes": {"ip": "10.128.0.9", "port": "22"},
                        "requestAttributes": {"auth": {}, "time": "2025-05-27T16:46:46.500915047Z"},
                    },
                    "resourceName": "1234567890123456789",
                    "serviceName": "iap.googleapis.com",
                    "status": {},
                },
                "receiveTimestamp": "2025-05-27 16:46:48.028847516",
                "resource": {
                    "labels": {
                        "instance_id": "1234567890123456789",
                        "project_id": "example-project",
                        "zone": "us-central1-f",
                    },
                    "type": "gce_instance",
                },
                "severity": "INFO",
                "timestamp": "2025-05-27 16:46:46.485356507",
            },
        ),
        RuleTest(
            name="SSH From OS Login Without MFA",
            expected_result=True,
            log={
                "p_any_emails": ["user@example.com"],
                "p_any_usernames": ["user"],
                "p_event_time": "2025-05-27 21:12:08.749558000",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2025-05-27 21:15:21.194027520",
                "p_row_id": "0a634c6bfeb190d7f3b2a9e326c0d10b",
                "p_schema_version": 0,
                "p_source_id": "bd7da315-647e-4eca-bcfe-083fab18f3f1",
                "p_source_label": "gcp-logsource",
                "p_udm": {"user": {"email": "user@example.com"}},
                "insertId": "fkz9lkf10luao",
                "labels": {"instance_id": "1234567890123456789", "zone": "us-central1-f"},
                "logName": "projects/example-project/logs/cloudaudit.googleapis.com%2Fdata_access",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@example.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "resource": "projects/example-project/zones/us-central1-f/instances/example-instance",
                        },
                    ],
                    "methodName": "google.cloud.oslogin.dataplane.OsLoginDataPlaneService.CheckPolicy",
                    "request": {
                        "@type": "type.googleapis.com/google.cloud.oslogin.dataplane.CheckPolicyRequest",
                        "email": "123456789012345678901",
                        "instance": "example-instance",
                        "numericProjectId": "123456789012",
                        "policy": "ADMIN_LOGIN",
                        "projectId": "example-project",
                        "serviceAccount": "example-sa@example-project.iam.gserviceaccount.com",
                        "zone": "us-central1-f",
                    },
                    "resourceName": "projects/example-project/zones/us-central1-f/instances/example-instance",
                    "response": {
                        "@type": "type.googleapis.com/google.cloud.oslogin.dataplane.CheckPolicyResponse",
                        "success": True,
                    },
                    "serviceName": "oslogin.googleapis.com",
                },
                "receiveTimestamp": "2025-05-27 21:12:09.376480048",
                "resource": {
                    "labels": {
                        "method": "google.cloud.oslogin.dataplane.OsLoginDataPlaneService.CheckPolicy",
                        "project_id": "example-project",
                        "service": "oslogin.googleapis.com",
                    },
                    "type": "audited_resource",
                },
                "severity": "INFO",
                "timestamp": "2025-05-27 21:12:08.749558000",
            },
        ),
        RuleTest(
            name="SSH From Remote Machine",
            expected_result=True,
            log={
                "p_any_ip_addresses": ["192.168.1.100"],
                "p_any_emails": ["user@example.com"],
                "p_any_usernames": ["user"],
                "p_event_time": "2025-05-27 15:18:22.406665000",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2025-05-27 15:20:21.064927161",
                "p_row_id": "6244057f4c79ddd1c9f8d8e226939706",
                "p_schema_version": 0,
                "p_source_id": "bd7da315-647e-4eca-bcfe-083fab18f3f1",
                "p_source_label": "gcp-logsource",
                "p_udm": {
                    "source": {"address": "192.168.1.100", "ip": "192.168.1.100"},
                    "user": {"email": "user@example.com"},
                },
                "insertId": "xwyto8dyr64",
                "labels": {"compute.googleapis.com/root_trigger_id": "9cff1773-d2d3-4e20-a605-3bf753bc9dc2"},
                "logName": "projects/example-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "id": "operation-1748359087846-6361f925ef323-87f8ece7-5a27a574",
                    "last": True,
                    "producer": "compute.googleapis.com",
                },
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {
                        "principalEmail": "user@example.com",
                        "principalSubject": "user:user@example.com",
                    },
                    "metadata": {
                        "@type": "type.googleapis.com/google.cloud.audit.GceProjectAuditMetadata",
                        "projectMetadataDelta": {"modifiedMetadataKeys": ["ssh-keys"]},
                    },
                    "methodName": "v1.compute.projects.setCommonInstanceMetadata",
                    "request": {"@type": "type.googleapis.com/compute.projects.setCommonInstanceMetadata"},
                    "requestMetadata": {
                        "callerIP": "192.168.1.100",
                        "callerSuppliedUserAgent": "example-user-agent",
                        "destinationAttributes": {},
                        "requestAttributes": {},
                    },
                    "resourceName": "projects/example-project",
                    "serviceName": "compute.googleapis.com",
                },
                "receiveTimestamp": "2025-05-27 15:18:23.188347406",
                "resource": {"labels": {"project_id": "123456789012"}, "type": "gce_project"},
                "severity": "NOTICE",
                "timestamp": "2025-05-27 15:18:22.406665000",
            },
        ),
        RuleTest(
            name="SSH From GCP Console",
            expected_result=True,
            log={
                "p_any_ip_addresses": ["192.168.1.100"],
                "p_any_emails": ["user@example.com"],
                "p_any_domain_names": ["www.googleapis.com"],
                "p_any_usernames": ["user"],
                "p_event_time": "2025-05-27 16:44:43.443688000",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2025-05-27 19:05:21.363614256",
                "p_row_id": "fee0f92d7864a191dfa994e326bb8604",
                "p_schema_version": 0,
                "p_source_id": "bd7da315-647e-4eca-bcfe-083fab18f3f1",
                "p_source_label": "gcp-logsource",
                "p_udm": {
                    "source": {"address": "192.168.1.100", "ip": "192.168.1.100"},
                    "user": {"email": "user@example.com"},
                },
                "insertId": "-6215lve1hil2",
                "labels": {"compute.googleapis.com/root_trigger_id": "f1c52ba7-0407-4ec5-b031-8f3232f828f0"},
                "logName": "projects/example-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "first": True,
                    "id": "operation-1748364283389-63620c80ca7f1-1f0f53ee-d1fa97bd",
                    "producer": "compute.googleapis.com",
                },
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {
                        "principalEmail": "user@example.com",
                        "principalSubject": "user:user@example.com",
                    },
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "compute.instances.setMetadata",
                            "resource": "projects/example-project/zones/us-central1-f/instances/example-instance",
                            "resourceAttributes": {
                                "name": "projects/example-project/zones/us-central1-f/instances/example-instance",
                                "service": "compute",
                                "type": "compute.instances",
                            },
                        },
                    ],
                    "metadata": {
                        "@type": "type.googleapis.com/google.cloud.audit.GceInstanceAuditMetadata",
                        "instanceMetadataDelta": {"addedMetadataKeys": ["ssh-keys"]},
                    },
                    "methodName": "v1.compute.instances.setMetadata",
                    "request": {"@type": "type.googleapis.com/compute.instances.setMetadata"},
                    "requestMetadata": {
                        "callerIP": "192.168.1.100",
                        "callerSuppliedUserAgent": "example-user-agent",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2025-05-27T16:44:43.801696Z"},
                    },
                    "resourceLocation": {"currentLocations": ["us-central1-f"]},
                    "resourceName": "projects/example-project/zones/us-central1-f/instances/example-instance",
                    "response": {
                        "@type": "type.googleapis.com/operation",
                        "id": "8662253207148844308",
                        "insertTime": "2025-05-27T09:44:43.751-07:00",
                        "name": "operation-1748364283389-63620c80ca7f1-1f0f53ee-d1fa97bd",
                        "operationType": "setMetadata",
                        "progress": "0",
                        "selfLink": "https://www.googleapis.com/compute/v1/projects/example-project/zones/us-central1-f/operations/operation-1748364283389-63620c80ca7f1-1f0f53ee-d1fa97bd",
                        "selfLinkWithId": "https://www.googleapis.com/compute/v1/projects/example-project/zones/us-central1-f/operations/8662253207148844308",
                        "startTime": "2025-05-27T09:44:43.768-07:00",
                        "status": "RUNNING",
                        "targetId": "1234567890123456789",
                        "targetLink": "https://www.googleapis.com/compute/v1/projects/example-project/zones/us-central1-f/instances/example-instance",
                        "user": "user@example.com",
                        "zone": "https://www.googleapis.com/compute/v1/projects/example-project/zones/us-central1-f",
                    },
                    "serviceName": "compute.googleapis.com",
                },
                "receiveTimestamp": "2025-05-27 16:44:44.041407158",
                "resource": {
                    "labels": {
                        "instance_id": "1234567890123456789",
                        "project_id": "example-project",
                        "zone": "us-central1-f",
                    },
                    "type": "gce_instance",
                },
                "severity": "NOTICE",
                "timestamp": "2025-05-27 16:44:43.443688000",
            },
        ),
        RuleTest(
            name="Connect to Serial Port",
            expected_result=True,
            log={
                "p_any_ip_addresses": ["192.168.1.100"],
                "p_event_time": "2025-05-27 21:27:27.208503055",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2025-05-27 21:30:21.036346031",
                "p_row_id": "ea1fcc08c947a6d0eab1a9e326faa309",
                "p_schema_version": 0,
                "p_source_id": "bd7da315-647e-4eca-bcfe-083fab18f3f1",
                "p_source_label": "gcp-logsource",
                "p_udm": {"source": {"address": "192.168.1.100", "ip": "192.168.1.100"}},
                "insertId": "199ms9obf4",
                "logName": "projects/example-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "first": True,
                    "id": "1996ae7d794305b3adac0ef09924deb467656fd2",
                    "producer": "us-central1-ssh-serialport.googleapis.com",
                },
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "methodName": "google.ssh-serialport.v1.connect",
                    "request": {
                        "@type": "type.googleapis.com/google.compute.SerialConsoleSessionBegin",
                        "serialConsoleOptions": [
                            {"name": "port", "value": "1"},
                            {"name": "source", "value": "pantheon"},
                        ],
                        "username": "user_example_com",
                    },
                    "requestMetadata": {"callerIP": "192.168.1.100"},
                    "resourceLocation": {"currentLocations": ["us-central1"], "originalLocations": ["us-central1"]},
                    "resourceName": "projects/example-project/zones/us-central1-f/instances/example-instance/SerialPort/1",
                    "serviceName": "us-central1-ssh-serialport.googleapis.com",
                    "status": {"message": "Connection succeeded."},
                },
                "receiveTimestamp": "2025-05-27 21:27:27.524944715",
                "resource": {
                    "labels": {
                        "instance_id": "1234567890123456789",
                        "project_id": "example-project",
                        "zone": "us-central1-f",
                    },
                    "type": "gce_instance",
                },
                "severity": "NOTICE",
                "timestamp": "2025-05-27 21:27:27.208503055",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.mongodb import mongodb_alert_context


@panther_managed
class MongoDBAtlasApiKeyCreated(Rule):
    default_description = "A MongoDB Atlas api key's access list was updated"
    display_name = "MongoDB Atlas API Key Created"
    default_severity = Severity.MEDIUM
    default_reference = "https://www.mongodb.com/docs/atlas/configure-api-access/#std-label-about-org-api-keys"
    log_types = [LogType.MONGODB_ORGANIZATION_EVENT]
    id = "MongoDB.Atlas.ApiKeyCreated-prototype"

    def rule(self, event):
        return event.get("eventTypeName", "") == "API_KEY_ACCESS_LIST_ENTRY_ADDED"

    def title(self, event):
        user = event.get("username", "<USER_NOT_FOUND>")
        public_key = event.get("targetPublicKey", "<PUBLIC_KEY_NOT_FOUND>")
        return f"MongoDB Atlas: [{user}] updated the allowed access list for API Key [{public_key}]"

    def alert_context(self, event):
        context = mongodb_alert_context(event)
        links = event.deep_walk("links", "href", return_val="first", default="<LINKS_NOT_FOUND>")
        extra_context = {
            "links": links,
            "event_type_name": event.get("eventTypeName", "<EVENT_TYPE_NOT_FOUND>"),
            "target_public_key": event.get("targetPublicKey", "<PUBLIC_KEY_NOT_FOUND>"),
        }
        context.update(extra_context)
        return context

    tests = [
        RuleTest(
            name="API Key Deleted",
            expected_result=False,
            log={
                "created": "2023-06-14 15:47:15",
                "currentvalue": {},
                "eventtypename": "API_KEY_ACCESS_LIST_ENTRY_DELETED",
                "id": "1234abcd13f2804962409423",
                "isglobaladmin": False,
                "links": [
                    {
                        "href": "https://cloud.mongodb.com/api/atlas/v1.0/orgs/9876xyz123lmnop0/events/1234abcd13f2804962409423",
                        "rel": "self",
                    },
                ],
                "orgid": "9876xyz123lmnop0",
                "p_event_time": "2023-06-14 15:47:15",
                "p_log_type": "MongoDB.OrganizationEvent",
                "p_parse_time": "2023-06-14 15:53:42.415",
                "p_row_id": "5ad9c2df49e19aac98def9e118e236",
                "p_schema_version": 0,
                "p_source_id": "a2e8f928-c6e5-4110-b6f9-1b741176041d",
                "p_source_label": "mongo-test-2",
                "remoteaddress": "1.2.3.4",
                "targetpublickey": "xfvcfwtt",
                "userid": "abcd1234userid988",
                "username": "user@company.com",
                "whitelistentry": "1.2.3.4",
            },
        ),
        RuleTest(
            name="API Key Created",
            expected_result=True,
            log={
                "created": "2023-06-14 15:47:15",
                "currentvalue": {},
                "eventtypename": "API_KEY_ACCESS_LIST_ENTRY_ADDED",
                "id": "1234abcd13f2804962409423",
                "isglobaladmin": False,
                "links": [
                    {
                        "href": "https://cloud.mongodb.com/api/atlas/v1.0/orgs/9876xyz123lmnop0/events/1234abcd13f2804962409423",
                        "rel": "self",
                    },
                ],
                "orgid": "9876xyz123lmnop0",
                "p_event_time": "2023-06-14 15:47:15",
                "p_log_type": "MongoDB.OrganizationEvent",
                "p_parse_time": "2023-06-14 15:53:42.415",
                "p_row_id": "5ad9c2df49e19aac98def9e118e236",
                "p_schema_version": 0,
                "p_source_id": "a2e8f928-c6e5-4110-b6f9-1b741176041d",
                "p_source_label": "mongo-test-2",
                "remoteaddress": "1.2.3.4",
                "targetpublickey": "xfvcfwtt",
                "userid": "abcd1234userid988",
                "username": "user@company.com",
                "whitelistentry": "1.2.3.4",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSConfigServiceDisabledDeleted(Rule):
    id = "AWS.ConfigService.DisabledDeleted-prototype"
    display_name = "AWS Config Service Disabled"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "Defense Evasion:Impair Defenses"]
    reports = {"CIS": ["3.9"], "MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.MEDIUM
    default_description = "An AWS Config Recorder or Delivery Channel was disabled or deleted\n"
    default_runbook = "Verify that the Config Service changes were authorized. If not, revert them and investigate who caused the change. Consider altering permissions to prevent this from happening again in the future.\n"
    default_reference = "https://aws.amazon.com/config/"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    # API calls that are indicative of an AWS Config Service change
    CONFIG_SERVICE_DISABLE_DELETE_EVENTS = {"StopConfigurationRecorder", "DeleteDeliveryChannel"}

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") in self.CONFIG_SERVICE_DISABLE_DELETE_EVENTS

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Config Recorder Delivery Channel Created",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "config.amazonaws.com",
                "eventName": "PutDeliveryChannel",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"configurationRecorderName": "default"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Config Recorder Deleted",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "config.amazonaws.com",
                "eventName": "DeleteDeliveryChannel",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"configurationRecorderName": "default"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Error Deleting Config Recorder",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "errorCode": "NoSuchDeliveryChannelException",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "config.amazonaws.com",
                "eventName": "DeleteDeliveryChannel",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"configurationRecorderName": "default"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther.rules.duo.duo_admin_app_integration_secret_key_viewed import (
    DuoAdminAppIntegrationSecretKeyViewed as DuoAdminAppIntegrationSecretKeyViewed,
)
from pypanther.rules.duo.duo_admin_bypass_code_created import DuoAdminBypassCodeCreated as DuoAdminBypassCodeCreated
from pypanther.rules.duo.duo_admin_bypass_code_viewed import DuoAdminBypassCodeViewed as DuoAdminBypassCodeViewed
from pypanther.rules.duo.duo_admin_create_admin import DuoAdminCreateAdmin as DuoAdminCreateAdmin
from pypanther.rules.duo.duo_admin_lockout import DuoAdminLockout as DuoAdminLockout
from pypanther.rules.duo.duo_admin_marked_push_fraudulent import (
    DUOAdminActionMarkedFraudulent as DUOAdminActionMarkedFraudulent,
)
from pypanther.rules.duo.duo_admin_mfa_restrictions_updated import (
    DuoAdminMFARestrictionsUpdated as DuoAdminMFARestrictionsUpdated,
)
from pypanther.rules.duo.duo_admin_new_admin_api_app_integration import (
    DuoAdminNewAdminAPIAppIntegration as DuoAdminNewAdminAPIAppIntegration,
)
from pypanther.rules.duo.duo_admin_policy_updated import DuoAdminPolicyUpdated as DuoAdminPolicyUpdated
from pypanther.rules.duo.duo_admin_sso_saml_requirement_disabled import (
    DuoAdminSSOSAMLRequirementDisabled as DuoAdminSSOSAMLRequirementDisabled,
)
from pypanther.rules.duo.duo_admin_user_mfa_bypass_enabled import (
    DuoAdminUserMFABypassEnabled as DuoAdminUserMFABypassEnabled,
)
from pypanther.rules.duo.duo_user_action_fraudulent import DUOUserActionFraudulent as DUOUserActionFraudulent
from pypanther.rules.duo.duo_user_anomalous_push import DUOUserDeniedAnomalousPush as DUOUserDeniedAnomalousPush
from pypanther.rules.duo.duo_user_bypass_code_used import DUOUserBypassCodeUsed as DUOUserBypassCodeUsed
from pypanther.rules.duo.duo_user_endpoint_failure_multi import DUOUserEndpointFailure as DUOUserEndpointFailure



"""
Used to define global helpers and variables for Box events.
"""

import json


# 'additional_details' from box logs varies by event_type.
# This helper wraps the process of extracting those details.
def box_parse_additional_details(event):
    additional_details = event.get("additional_details", {})
    if isinstance(additional_details, (str, bytes)):
        try:
            return json.loads(additional_details)
        except ValueError:
            return {}
    return additional_details



import argparse
import pathlib

from pypanther import display, generate, get_rule, list_log_types, list_rules, shared_args, upload
from pypanther.backend import util
from pypanther.utils import parse_bool_input

DEFAULT_SCHEMAS_PATH = "content/schemas/"


def setup_list_rules_parser(list_rules_parser: argparse.ArgumentParser):
    list_rules_parser.set_defaults(func=list_rules.run)
    shared_args.for_filtering(list_rules_parser)
    list_rules_parser.add_argument(
        "--managed",
        help="Filter by Panther managed or non-Panther managed rules",
        default=None,
        required=False,
        type=parse_bool_input,
    )
    list_rules_parser.add_argument(
        "--attributes",
        help="Display attributes of rules as columns in printed table (i.e --attributes threshold default_display_name). "
        f"Use '{display.ALL_TABLE_ATTR}' to display all attributes.",
        nargs="+",
        default=display.DEFAULT_RULE_TABLE_ATTRS,
        required=False,
        choices=display.VALID_RULE_TABLE_ATTRS + [display.ALL_TABLE_ATTR],
    )
    list_rules_parser.add_argument(
        "--sort-by",
        help="Choose a field to sort the output by.",
        default=display.DEFAULT_RULE_TABLE_SORT_BY,
        required=False,
        choices=display.VALID_RULE_TABLE_ATTRS,
    )
    list_rules_parser.add_argument(
        "--output",
        help="The format to use for the output.",
        required=False,
        choices=display.COMMON_CLI_OUTPUT_TYPES + [display.OUTPUT_TYPE_CSV],
        default=display.DEFAULT_CLI_OUTPUT_TYPE,
    )


def setup_get_rule_parser(get_rules_parser: argparse.ArgumentParser):
    get_rules_parser.set_defaults(func=get_rule.run)
    get_rules_parser.add_argument(
        "id",
        help="Required. The id of the rule to get",
        type=str,
    )
    get_rules_parser.add_argument(
        "--output",
        help="The format to use for the output.",
        required=False,
        choices=display.COMMON_CLI_OUTPUT_TYPES,
        default=display.DEFAULT_CLI_OUTPUT_TYPE,
    )
    get_rules_parser.add_argument(
        "--original",
        help="Return the original class definition instead of the final versions of the attributes",
        required=False,
        default=False,
        action="store_true",
    )


def setup_test_parser(test_parser: argparse.ArgumentParser):
    shared_args.for_filtering(test_parser)
    test_parser.add_argument(
        "--test-names",
        help="The names of the tests to run, space delimited. If not specified, all tests on filtered items will be run",
        nargs="+",
        default=None,
        required=False,
    )
    test_parser.add_argument(
        "--verbose",
        help="Verbose output, includes passing tests, skipped tests, and exception stack traces",
        default=False,
        required=False,
        action="store_true",
    )
    test_parser.add_argument(
        "--output",
        help="The format to use for the output.",
        required=False,
        choices=display.COMMON_CLI_OUTPUT_TYPES,
        default=display.DEFAULT_CLI_OUTPUT_TYPE,
    )


def setup_upload_parser(upload_parser: argparse.ArgumentParser):
    upload_parser.set_defaults(func=util.func_with_backend(upload.run))
    upload_parser.add_argument(
        "--skip-tests",
        help="Skip running tests and go directly to upload",
        default=False,
        required=False,
        action="store_true",
    )
    upload_parser.add_argument(
        "--verbose",
        help="Verbose output",
        default=False,
        required=False,
        action="store_true",
    )
    upload_parser.add_argument(
        "--output",
        help="The format to use for the output.",
        required=False,
        choices=display.COMMON_CLI_OUTPUT_TYPES,
        default=display.DEFAULT_CLI_OUTPUT_TYPE,
    )
    upload_parser.add_argument(
        "--schemas-path",
        help="Path to the schemas directory",
        default=DEFAULT_SCHEMAS_PATH,
        required=False,
    )
    dry_run_group = upload_parser.add_mutually_exclusive_group()
    dry_run_group.add_argument(
        "--skip-summary",
        help="Omit changes summary in output",
        default=False,
        required=False,
        action="store_true",
    )
    dry_run_group.add_argument(
        "--dry-run",
        help="Avoid actually uploading",
        default=False,
        required=False,
        action="store_true",
    )


def setup_list_log_types_parser(list_log_types_parser: argparse.ArgumentParser):
    list_log_types_parser.set_defaults(func=list_log_types.run)
    list_log_types_parser.add_argument(
        "substring",
        nargs="?",
        help="Filter log types by a substring. Only log types that include the substring will be printed.",
    )
    list_log_types_parser.add_argument(
        "--custom-only",
        help="Output only the custom log-types defined locally",
        default=False,
        required=False,
        action="store_true",
    )
    list_log_types_parser.add_argument(
        "--schemas-path",
        help="Path to the schemas directory",
        default=DEFAULT_SCHEMAS_PATH,
        required=False,
    )
    list_log_types_parser.add_argument(
        "--output",
        help="The format to use for the output.",
        required=False,
        choices=display.COMMON_CLI_OUTPUT_TYPES,
        default=display.DEFAULT_CLI_OUTPUT_TYPE,
    )


def setup_convert_parser(convert_parser: argparse.ArgumentParser):
    convert_parser.set_defaults(
        func=generate.convert,
        keep_all_rules=False,
        cwd_must_be_empty=True,
    )
    convert_parser.add_argument(
        "--verbose",
        help="Verbose output",
        default=False,
        required=False,
        action="store_true",
    )
    convert_parser.add_argument(
        "--pypanther-directory-name",
        help="The name that will be used for the top level directory where the converted artifacts will be placed",
        default="content",
        required=False,
    )
    convert_parser.add_argument(
        "panther_analysis_path",
        help="Path to the Panther Analysis directory",
        type=pathlib.Path,
    )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.okta import okta_alert_context


@panther_managed
class OktaUserReportedSuspiciousActivity(Rule):
    default_description = "Suspicious Activity Reporting provides an end user with the option to report unrecognized activity from an account activity email notification.\nThis detection alerts when a user marks the raised activity as suspicious."
    default_reference = "https://help.okta.com/en-us/Content/Topics/Security/suspicious-activity-reporting.htm"
    display_name = "Okta User Reported Suspicious Activity"
    default_severity = Severity.HIGH
    log_types = [LogType.OKTA_SYSTEM_LOG]
    id = "Okta.User.Reported.Suspicious.Activity-prototype"

    def rule(self, event):
        return event.get("eventtype") == "user.account.report_suspicious_activity_by_enduser"

    def title(self, event):
        reported_event_type = (
            event.get("debugcontext", {})
            .get("debugData", {})
            .get("suspiciousActivityEventType", "<event-type-not-found>")
        )
        return f"Okta: [{event.get('actor', {}).get('alternateId', '<id-not-found>')}] reported suspicious account activity [{reported_event_type}]."

    def alert_context(self, event):
        return okta_alert_context(event)

    tests = [
        RuleTest(
            name="Other Event",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 20, "lon": -25},
                        "postalCode": "12345",
                        "state": "Ohio",
                    },
                    "ipAddress": "1.3.2.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "requestId": "AbCdEf12G",
                        "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                        "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                    },
                },
                "displaymessage": "Reset all factors for user",
                "eventtype": "user.mfa.factor.reset_all",
                "legacyeventtype": "core.user.factor.reset_all",
                "outcome": {"result": "SUCCESS"},
                "published": "2022-06-22 18:18:29.015",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Springfield",
                                "country": "United States",
                                "geolocation": {"lat": 20, "lon": -25},
                                "postalCode": "12345",
                                "state": "Ohio",
                            },
                            "ip": "1.3.2.4",
                            "version": "V4",
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 701,
                    "asOrg": "verizon",
                    "domain": "verizon.net",
                    "isProxy": False,
                    "isp": "verizon",
                },
                "severity": "INFO",
                "target": [
                    {
                        "alternateId": "peter.griffin@company.com",
                        "displayName": "Peter Griffin",
                        "id": "0002222AAAA",
                        "type": "User",
                    },
                ],
                "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
                "uuid": "AbC-123-XyZ",
                "version": "0",
            },
        ),
        RuleTest(
            name="Suspicious Report Event",
            expected_result=True,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00ABC123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "aaa1234"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 30, "lon": -55},
                        "postalCode": "12345",
                        "state": "Texas",
                    },
                    "ipAddress": "1.2.3.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "dtHash": "ab123542525",
                        "requestId": "A1S05S00000",
                        "requestUri": "/api/internal/users/me/report-suspicious-activity",
                        "suspiciousActivityBrowser": "SAFARI",
                        "suspiciousActivityEventCity": "Springfield",
                        "suspiciousActivityEventCountry": "United States",
                        "suspiciousActivityEventId": "aaa-123-bbb",
                        "suspiciousActivityEventIp": "9.8.7.6",
                        "suspiciousActivityEventLatitude": "30.000",
                        "suspiciousActivityEventLongitude": "-55.000",
                        "suspiciousActivityEventState": "Texas",
                        "suspiciousActivityEventTransactionId": "ABC12345",
                        "suspiciousActivityEventType": "system.email.new_device_notification.sent_message",
                        "suspiciousActivityOs": "Mac OS X (iPhone)",
                        "suspiciousActivityTimestamp": "2022-12-14T15:58:50.347Z",
                        "url": "/api/internal/users/me/report-suspicious-activity?i=aaaaaa",
                    },
                },
                "displaymessage": "User report suspicious activity",
                "eventtype": "user.account.report_suspicious_activity_by_enduser",
                "legacyeventtype": "core.user.account.report_suspicious_activity_by_enduser",
                "outcome": {"result": "SUCCESS"},
                "published": "2022-12-14 15:58:58.851",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Austin",
                                "country": "United States",
                                "geolocation": {"lat": 30, "lon": -55},
                                "postalCode": "12345",
                                "state": "Texas",
                            },
                            "ip": "9.8.7.6",
                            "version": "V4",
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 11427,
                    "asOrg": "charter communications inc",
                    "domain": "spectrum.com",
                    "isProxy": False,
                    "isp": "charter communications inc",
                },
                "severity": "WARN",
                "target": [
                    {
                        "alternateId": "homer.simpson@duff.com",
                        "displayName": "Homer Simpson",
                        "id": "01234",
                        "type": "User",
                    },
                ],
                "transaction": {"detail": {}, "id": "1234ABC", "type": "WEB"},
                "uuid": "ABC1234",
                "version": "0",
            },
        ),
    ]



"""Used to simplify the use of MISP Data in Rules"""

from typing import Union

from pypanther.helpers.lookuptable import LookupTableMatches

MISP_WARNING_LISTS_LUT_NAME = "misp_warning_lists"


class MISPWarningLists(LookupTableMatches):
    """Helper to get MISP Warning Lists information for enriched fields"""

    def __init__(self, event):
        super().__init__()
        super()._register(event, MISP_WARNING_LISTS_LUT_NAME)

    def warning_lists(self, match_field: str) -> Union[list, None]:
        """Get warning lists for the matched IP"""
        return self._lookup(match_field, "warning_lists")

    def cidr(self, match_field: str) -> Union[list[str], str]:
        """Get CIDR range for the matched IP"""
        return self._lookup(match_field, "cidr")

    def has_warning_list_id(self, source_ip: str, warning_list_id: str) -> bool:
        """Check if the source IP has a specific warning list ID"""
        # MISP data is structured as: MISP Warning Lists -> p_any_ip_addresses -> [array of matches]
        misp_data = self._lookup("p_any_ip_addresses")
        if not misp_data:
            return False

        # misp_data should be a list of IP match objects
        for ip_match in misp_data:
            if ip_match.get("p_match") == source_ip:
                warning_lists = ip_match.get("warning_lists", [])
                if any(wl.get("id") == warning_list_id for wl in warning_lists):
                    return True

        return False


def get_misp_warning_lists(event):
    """Returns a MISPWarningLists object for the event or None if it is not available"""
    if event.deep_get("p_enrichment", MISP_WARNING_LISTS_LUT_NAME):
        return MISPWarningLists(event)
    return None
